import logging

from openfisca_france.model.base import *


log = logging.getLogger(__name__)


# Csg déductible
class f6de(Variable):
    cerfa_field = '6DE'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'CSG déductible calculée sur les revenus du patrimoine'
    definition_period = YEAR


# Pensions alimentaires
class f6gi(Variable):
    cerfa_field = '6GI'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Pensions alimentaires versées à des enfants majeurs (décision de justice définitive avant 2006): 1er enfant'
    definition_period = YEAR


class f6gj(Variable):
    cerfa_field = '6GJ'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Pensions alimentaires versées à des enfants majeurs (décision de justice définitive avant 2006): 2eme enfant'
    definition_period = YEAR


class f6el(Variable):
    cerfa_field = '6EL'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Autres pensions alimentaires versées à des enfants majeurs: 1er enfant'
    # start_date = date(2006, 1, 1)
    definition_period = YEAR


class f6em(Variable):
    cerfa_field = '6EM'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Autres pensions alimentaires versées à des enfants majeurs: 2eme enfant'
    # start_date = date(2006, 1, 1)
    definition_period = YEAR


class f6gp(Variable):
    cerfa_field = '6GP'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Autres pensions alimentaires versées décision de justice définitive avant 2006 (mineurs, ascendants)'
    definition_period = YEAR


class f6gu(Variable):
    cerfa_field = '6GU'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Autres pensions alimentaires versées (mineurs, ascendants)'
    # start_date = date(2006, 1, 1)
    definition_period = YEAR


# Frais d'accueil d'une personne de plus de 75 ans dans le besoin
class f6eu(Variable):
    cerfa_field = '6EU'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = "Frais d'accueil de personnes de plus de 75 ans dans le besoin"
    definition_period = YEAR


class f6ev(Variable):
    cerfa_field = '6EV'
    value_type = int
    is_period_size_independent = True
    entity = FoyerFiscal
    label = 'Nombre de personnes de plus de 75 ans dans le besoin accueillies sous votre toit'
    definition_period = YEAR


# Déductions diverses
class f6dd(Variable):
    cerfa_field = '6DD'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Déductions diverses'
    definition_period = YEAR


# Épargne retraite - PERP, PRÉFON, COREM et CGOS
class f6ps(Variable):
    cerfa_field = {
        0: '6PS',
        1: '6PT',
        2: '6PU',
        }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Plafond de déduction épargne retraite (plafond calculé sur les revenus perçus en n-1)'
    definition_period = YEAR


class f6rs(Variable):
    cerfa_field = {
        0: '6RS',
        1: '6RT',
        2: '6RU',
        }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = "Cotisations d'épargne retraite versées au titre d'un PERP, PREFON, COREM et C.G.O.S"
    definition_period = YEAR


class f6ss(Variable):
    cerfa_field = {
        0: '6SS',
        1: '6ST',
        2: '6SU',
        }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Rachat de cotisations PERP, PREFON, COREM et C.G.O.S'
    definition_period = YEAR


# Souscriptions en faveur du cinéma ou de l’audiovisuel
# TODO: ancien numéro de case, antérieur à 2008 ....au moins! vérifier pour 07-06-05 ect...probablement avant 2005 (autre nom en 12 et 13)
class f6aa(Variable):
    cerfa_field = '6AA'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Souscriptions en faveur du cinéma ou de l’audiovisuel'
    # start_date = date(2005, 1, 1)
    end = '2006-12-31'
    definition_period = YEAR


# Souscriptions au capital des SOFIPÊCHE


# ancien numéro de case, antérieur à 2008 ....au moins vérifier pour 07-06-05 ect...probablement avant 2005 (autre nom en  12 et13)
class f6cc(Variable):
    cerfa_field = 'CC'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Souscriptions au capital des SOFIPÊCHE'
    # start_date = date(2005, 1, 1)
    end = '2005-12-31'
    definition_period = YEAR


# Investissements DOM-TOM dans le cadre d’une entreprise < = 2005
# ou Versements sur un compte épargne codéveloppement
class f6eh(Variable):
    cerfa_field = 'EH'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    # start_date = date(2005, 1, 1)
    end = '2005-12-31'
    definition_period = YEAR
# TODO: vérifier date de début et de fin de cette case (rien en 12 et 13)


class f6da(Variable):
    cerfa_field = 'DA'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Pertes en capital consécutives à la souscription au capital de sociétés nouvelles ou de sociétés en difficulté'
    # start_date = date(2005, 1, 1)
    end = '2005-12-31'
    definition_period = YEAR


# Dépenses de grosses réparations effectuées par les nus propriétaires
class f6cb(Variable):
    cerfa_field = '6CB'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = "Dépenses de grosses réparations effectuées par les nus-propriétaires (dépenses réalisées au cours de l'année de perception des revenus)"
    end = '2017-12-31'
    definition_period = YEAR


# TODO: before 2006 was Pertes en capital consécutives à la souscription au capital de sociétés nouvelles ou de sociétés en difficulté (cases CB et DA de la déclaration complémentaire)

class f6hj(Variable):
    cerfa_field = '6HJ'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = "Dépenses de grosses réparations effectuées par les nus-propriétaires: report des dépenses de l'année 2009"
    end = '2019-12-31'
    definition_period = YEAR


class f6hk(Variable):
    cerfa_field = '6HK'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = "Dépenses de grosses réparations effectuées par les nus-propriétaires: report des dépenses de l'année 2010"
    # start_date = date(2011, 1, 1)
    definition_period = YEAR


class f6hl(Variable):
    cerfa_field = '6HL'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = "Dépenses de grosses réparations effectuées par les nus-propriétaires: report des dépenses de l'année 2011"
    # start_date = date(2012, 1, 1)
    definition_period = YEAR


class f6hm(Variable):
    cerfa_field = '6HM'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = "Dépenses de grosses réparations effectuées par les nus-propriétaires: report des dépenses de l'année 2012"
    # start_date = date(2013, 1, 1)
    definition_period = YEAR


class f6hn(Variable):
    cerfa_field = '6HN'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = "Dépenses de grosses réparations effectuées par les nus-propriétaires: report des dépenses de l'année 2013"
    # start_date = date(2014, 1, 1)
    definition_period = YEAR


class f6ho(Variable):
    cerfa_field = '6HO'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = "Dépenses de grosses réparations effectuées par les nus-propriétaires: report des dépenses de l'année 2014"
    # start_date = date(2015, 1, 1)
    definition_period = YEAR


class f6hp(Variable):
    cerfa_field = '6HP'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = "Dépenses de grosses réparations effectuées par les nus-propriétaires: report des dépenses de l'année 2015"
    # start_date = date(2016, 1, 1)
    definition_period = YEAR


class f6hq(Variable):
    cerfa_field = '6HQ'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = "Dépenses de grosses réparations effectuées par les nus-propriétaires: report des dépenses de l'année 2016"
    # start_date = date(2017, 1, 1)
    definition_period = YEAR


class f6hr(Variable):
    cerfa_field = '6HR'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = "Dépenses de grosses réparations effectuées par les nus-propriétaires: report des dépenses de l'année 2017"
    # start_date = date(2018, 1, 1)
    definition_period = YEAR


# Sommes à rajouter au revenu imposable
class f6gh(Variable):
    cerfa_field = '6GH'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Sommes à ajouter au revenu imposable'
    definition_period = YEAR


# Deficits antérieurs
class f6fa(Variable):
    cerfa_field = '6FA'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Deficits globaux des années antérieures non encore déduits les années précédentes: année de perception des revenus -6'
    definition_period = YEAR


class f6fb(Variable):
    cerfa_field = '6FB'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Deficits globaux des années antérieures non encore déduits: année de perception des revenus -5'
    definition_period = YEAR


class f6fc(Variable):
    cerfa_field = '6FC'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Deficits globaux des années antérieures non encore déduits: année de perception des revenus -4'
    definition_period = YEAR


class f6fd(Variable):
    cerfa_field = '6FD'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Deficits globaux des années antérieures non encore déduits: année de perception des revenus -3'
    definition_period = YEAR


class f6fe(Variable):
    cerfa_field = '6FE'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Deficits globaux des années antérieures non encore déduits: année de perception des revenus -2'
    definition_period = YEAR


class f6fl(Variable):
    cerfa_field = '6FL'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Deficits globaux des années antérieures non encore déduits: année de perception des revenus -1'
    definition_period = YEAR


class rfr_cd(Variable):
    value_type = float
    entity = FoyerFiscal
    label = 'Charges déductibles entrant dans le revenus fiscal de référence'
    reference = 'Article 1417 du Code Général des Impôts - IV-1°-a)'
    definition_period = YEAR

    def formula(foyer_fiscal, period, parameters):
        cd_eparet = foyer_fiscal('cd_eparet', period)
        cd_sofipe = foyer_fiscal('cd_sofipe', period)

        return cd_eparet + cd_sofipe

    def formula_2007_01_01(foyer_fiscal, period, parameters):
        cd_eparet = foyer_fiscal('cd_eparet', period)

        return cd_eparet


class cd1(Variable):
    value_type = float
    entity = FoyerFiscal
    label = 'Charges déductibles non plafonnées'
    reference = 'http://impotsurlerevenu.org/definitions/215-charge-deductible.php'
    definition_period = YEAR

    def formula_2002_01_01(foyer_fiscal, period, parameters):
        '''
        Renvoie la liste des charges déductibles avant rbg_int pour 2002
        '''
        pensions_alimentaires_deduites = foyer_fiscal('pensions_alimentaires_deduites', period)
        cd_acc75a = foyer_fiscal('cd_acc75a', period)
        pertes_capital_societes_nouvelles = foyer_fiscal('pertes_capital_societes_nouvelles', period)
        cd_deddiv = foyer_fiscal('cd_deddiv', period)
        cd_doment = foyer_fiscal('cd_doment', period)

        niches1 = pensions_alimentaires_deduites + cd_acc75a + pertes_capital_societes_nouvelles + cd_deddiv + cd_doment
        return niches1

    def formula_2004_01_01(foyer_fiscal, period, parameters):
        '''
        Renvoie la liste des charges déductibles avant rbg_int pour 2004
        '''
        pensions_alimentaires_deduites = foyer_fiscal('pensions_alimentaires_deduites', period)
        cd_acc75a = foyer_fiscal('cd_acc75a', period)
        pertes_capital_societes_nouvelles = foyer_fiscal('pertes_capital_societes_nouvelles', period)
        cd_deddiv = foyer_fiscal('cd_deddiv', period)
        cd_doment = foyer_fiscal('cd_doment', period)
        cd_eparet = foyer_fiscal('cd_eparet', period)

        niches1 = (
            pensions_alimentaires_deduites
            + cd_acc75a
            + pertes_capital_societes_nouvelles
            + cd_deddiv
            + cd_doment
            + cd_eparet
            )

        return niches1

    def formula_2006_01_01(foyer_fiscal, period, parameters):
        '''
        Renvoie la liste des charges déductibles avant rbg_int pour 2006
        '''
        pensions_alimentaires_deduites = foyer_fiscal('pensions_alimentaires_deduites', period)
        cd_acc75a = foyer_fiscal('cd_acc75a', period)
        pertes_capital_societes_nouvelles = foyer_fiscal('pertes_capital_societes_nouvelles', period)
        cd_deddiv = foyer_fiscal('cd_deddiv', period)
        cd_eparet = foyer_fiscal('cd_eparet', period)

        niches1 = pensions_alimentaires_deduites + cd_acc75a + pertes_capital_societes_nouvelles + cd_deddiv + cd_eparet
        return niches1

    def formula_2007_01_01(foyer_fiscal, period, parameters):
        '''
        Renvoie la liste des charges déductibles avant rbg_int pour 2007
        '''
        pensions_alimentaires_deduites = foyer_fiscal('pensions_alimentaires_deduites', period)
        cd_acc75a = foyer_fiscal('cd_acc75a', period)
        cd_deddiv = foyer_fiscal('cd_deddiv', period)
        cd_eparet = foyer_fiscal('cd_eparet', period)

        niches1 = pensions_alimentaires_deduites + cd_acc75a + cd_deddiv + cd_eparet
        return niches1

    def formula_2009_01_01(foyer_fiscal, period, parameters):
        '''
        Renvoie la liste des charges déductibles avant rbg_int pour 2009
        '''
        pensions_alimentaires_deduites = foyer_fiscal('pensions_alimentaires_deduites', period)
        cd_acc75a = foyer_fiscal('cd_acc75a', period)
        cd_deddiv = foyer_fiscal('cd_deddiv', period)
        cd_eparet = foyer_fiscal('cd_eparet', period)
        grosses_reparations = foyer_fiscal('grosses_reparations', period)

        niches1 = pensions_alimentaires_deduites + cd_acc75a + cd_deddiv + cd_eparet + grosses_reparations
        return niches1

    def formula_2014_01_01(foyer_fiscal, period, parameters):
        '''
        Renvoie la liste des charges déductibles avant rbg_int pour 2014
        '''
        pensions_alimentaires_deduites = foyer_fiscal('pensions_alimentaires_deduites', period)
        cd_acc75a = foyer_fiscal('cd_acc75a', period)
        cd_deddiv = foyer_fiscal('cd_deddiv', period)
        cd_eparet = foyer_fiscal('cd_eparet', period)
        grosses_reparations = foyer_fiscal('grosses_reparations', period)

        niches1 = pensions_alimentaires_deduites + cd_acc75a + cd_deddiv + cd_eparet + grosses_reparations
        # log.error("Charges déductibles to be checked because not defined for %s", 2014)
        return niches1


class cd2(Variable):
    value_type = float
    entity = FoyerFiscal
    label = 'Charges déductibles plafonnées'
    reference = 'http://impotsurlerevenu.org/definitions/215-charge-deductible.php'
    definition_period = YEAR
    end = '2008-12-31'

    def formula_2002_01_01(foyer_fiscal, period, parameters):
        '''
        Renvoie la liste des charges déductibles à intégrer après le rbg_int
        '''
        cd_sofipe = foyer_fiscal('cd_sofipe', period)
        cinema = foyer_fiscal('souscriptions_cinema_audiovisuel', period)

        niches2 = cd_sofipe + cinema
        return niches2

    def formula_2006_01_01(foyer_fiscal, period, parameters):
        '''
        Renvoie la liste des charges déductibles à intégrer après le rbg_int
        '''
        cd_sofipe = foyer_fiscal('cd_sofipe', period)

        niches2 = cd_sofipe
        return niches2

    def formula_2007_01_01(foyer_fiscal, period, parameters):
        '''
        Renvoie la liste des charges déductibles à intégrer après le rbg_int
        '''
        epargne_codeveloppement_deduc = foyer_fiscal('epargne_codeveloppement_deduc', period)

        niches2 = epargne_codeveloppement_deduc
        return niches2


class rbg_int(Variable):
    value_type = float
    entity = FoyerFiscal
    label = 'Revenu brut global intermédiaire'
    definition_period = YEAR

    def formula(foyer_fiscal, period, parameters):
        rbg = foyer_fiscal('rbg', period)
        cd1 = foyer_fiscal('cd1', period)

        return max_(rbg - cd1, 0)


class charges_deduc(Variable):
    value_type = float
    entity = FoyerFiscal
    label = 'Charges déductibles'
    reference = 'http://impotsurlerevenu.org/definitions/215-charge-deductible.php'
    definition_period = YEAR

    def formula(foyer_fiscal, period, parameters):
        cd1 = foyer_fiscal('cd1', period)
        cd2 = foyer_fiscal('cd2', period)

        return cd1 + cd2


class pensions_alimentaires_deduites(Variable):
    value_type = float
    entity = FoyerFiscal
    label = 'Pensions alimentaires'
    reference = 'http://frederic.anne.free.fr/Cours/ITV.htm'
    definition_period = YEAR

    def formula_2006_01_01(foyer_fiscal, period, parameters):
        f6gi = foyer_fiscal('f6gi', period)
        f6gj = foyer_fiscal('f6gj', period)
        f6gp = foyer_fiscal('f6gp', period)
        f6el = foyer_fiscal('f6el', period)
        f6em = foyer_fiscal('f6em', period)
        f6gu = foyer_fiscal('f6gu', period)
        penalim = parameters(period).impot_revenu.calcul_revenus_imposables.charges_deductibles.pensions_alimentaires

        max1 = penalim.plafond
        taux_jgt_2006 = penalim.taux_jgt_2006
        # TODO: si vous subvenez seul(e) à l'entretien d'un enfant marié ou
        # pacsé ou chargé de famille, quel que soit le nmbre d'enfants du jeune
        # foyer, la déduction est limitée à 2*max
        # S'il habite chez ses parents, max 3359, sinon 5698
        return (
            min_(f6gi * (1 + taux_jgt_2006), max1)
            + min_(f6gj * (1 + taux_jgt_2006), max1)
            + min_(f6el, max1)
            + min_(f6em, max1)
            + f6gp
            * (1 + taux_jgt_2006) + f6gu
            )


class cd_acc75a(Variable):
    value_type = float
    entity = FoyerFiscal
    label = 'Frais d’accueil sous votre toit d’une personne de plus de 75 ans'
    definition_period = YEAR

    def formula(foyer_fiscal, period, parameters):
        f6eu = foyer_fiscal('f6eu', period)
        f6ev = foyer_fiscal('f6ev', period)
        acc75a = parameters(period).impot_revenu.calcul_revenus_imposables.charges_deductibles.accueil_personne_agee
        amax = acc75a.plafond * max_(1, f6ev)
        return min_(f6eu, amax)


class pertes_capital_societes_nouvelles(Variable):
    value_type = float
    entity = FoyerFiscal
    label = 'Pertes en capital consécutives à la souscription au capital de sociétés nouvelles ou de sociétés en difficulté'
    definition_period = YEAR
    end = '2006-12-31'

    def formula_2002_01_01(foyer_fiscal, period, parameters):
        '''
        Pertes en capital consécutives à la souscription au capital de sociétés
        nouvelles ou de sociétés en difficulté (cases CB et DA de la déclaration
        complémentaire)
        '''
        f6cb = foyer_fiscal('f6cb', period)
        maries_ou_pacses = foyer_fiscal('maries_ou_pacses', period)
        percap = parameters(period).impot_revenu.calcul_revenus_imposables.charges_deductibles.pertes_en_capital_societes_nouvelles
        plafond_cb = percap.plafond_cb * (1 + maries_ou_pacses)
        return min_(f6cb, plafond_cb)

    def formula_2003_01_01(foyer_fiscal, period, parameters):
        '''
        Pertes en capital consécutives à la souscription au capital de sociétés
        nouvelles ou de sociétés en difficulté (cases CB et DA de la déclaration
        complémentaire)
        '''
        f6cb = foyer_fiscal('f6cb', period)
        f6da = foyer_fiscal('f6da', period)
        maries_ou_pacses = foyer_fiscal('maries_ou_pacses', period)
        percap = parameters(period).impot_revenu.calcul_revenus_imposables.charges_deductibles.pertes_en_capital_societes_nouvelles
        plafond_cb = percap.plafond_cb * (1 + maries_ou_pacses)
        plafond_da = percap.plafond_da * (1 + maries_ou_pacses)
        return min_(min_(f6cb, plafond_cb) + min_(f6da, plafond_da), plafond_da)


class cd_deddiv(Variable):
    value_type = float
    entity = FoyerFiscal
    label = 'Déductions diverses'
    definition_period = YEAR

    def formula(foyer_fiscal, period, parameters):
        f6dd = foyer_fiscal('f6dd', period)

        return f6dd


class cd_doment(Variable):
    value_type = float
    entity = FoyerFiscal
    label = 'Investissements DOM-TOM dans le cadre d’une entreprise'
    end = '2005-12-31'
    definition_period = YEAR

    def formula_2002(foyer_fiscal, period, parameters):
        '''
        Investissements DOM-TOM dans le cadre d’une entreprise (case EH de la
        déclaration n° 2042 complémentaire)
        '''
        f6eh = foyer_fiscal('f6eh', period)

        return f6eh


class cd_eparet(Variable):
    value_type = float
    entity = FoyerFiscal
    label = "Charge déductible au titre de l'épargne retraite (PERP, PRÉFON, COREM et CGOS)"
    definition_period = YEAR

    def formula_2004(foyer_fiscal, period, parameters):
        f6ps_i = foyer_fiscal.members('f6ps', period)
        f6rs_i = foyer_fiscal.members('f6rs', period)
        f6ss_i = foyer_fiscal.members('f6ss', period)

        # TODO: En théorie, les plafonds de déductions (ps, pt, pu) sont calculés sur
        # le formulaire 2041 GX
        return foyer_fiscal.sum(
            where(
                f6ps_i == 0,
                f6rs_i + f6ss_i,
                min_(f6rs_i + f6ss_i, f6ps_i)
                )
            )


class cd_sofipe(Variable):
    value_type = float
    entity = FoyerFiscal
    label = 'Souscriptions au capital des SOFIPÊCHE'
    end = '2006-12-31'
    definition_period = YEAR

    def formula_2002(foyer_fiscal, period, parameters):
        '''
        Souscriptions au capital des SOFIPÊCHE (case CC de la déclaration
        complémentaire)
        '''
        f6cc = foyer_fiscal('f6cc', period)
        rbg_int = foyer_fiscal('rbg_int', period)
        maries_ou_pacses = foyer_fiscal('maries_ou_pacses', period)
        sofipeche = parameters(period).impot_revenu.calcul_revenus_imposables.charges_deductibles.deductions_souscription_parts_sofipeche

        plafond = min_(sofipeche.plafond_en_revenu_net_global * rbg_int, sofipeche.plafond * (1 + maries_ou_pacses))
        return min_(f6cc, plafond)


class souscriptions_cinema_audiovisuel(Variable):
    value_type = float
    entity = FoyerFiscal
    label = 'Souscriptions en faveur du cinéma ou de l’audiovisuel'
    end = '2005-12-31'
    definition_period = YEAR

    def formula_2002(foyer_fiscal, period, parameters):
        '''
        Souscriptions en faveur du cinéma ou de l’audiovisuel (case AA de la
        déclaration n° 2042 complémentaire)
        '''
        f6aa = foyer_fiscal('f6aa', period)
        rbg_int = foyer_fiscal('rbg_int', period)
        cinema = parameters(period).impot_revenu.calcul_revenus_imposables.charges_deductibles.cinema

        max1 = min_(cinema.taux * rbg_int, cinema.plafond)
        return min_(f6aa, max1)


class epargne_codeveloppement_deduc(Variable):
    value_type = float
    entity = FoyerFiscal
    label = 'Versements sur un compte épargne codéveloppement'
    end = '2008-12-31'
    # En 2009, on passe d'un montant déductible à une réduction d'impôt : voir reductions_impot_plafonnees.py
    definition_period = YEAR

    def formula_2007(foyer_fiscal, period, parameters):
        '''
        Versements sur un compte épargne codéveloppement (case EH de la déclaration
        complémentaire)
        '''
        f6eh = foyer_fiscal('f6eh', period)
        rbg_int = foyer_fiscal('rbg_int', period)
        codev = parameters(period).impot_revenu.calcul_reductions_impots.compte_epargne_co_developpement

        plafond = min_(codev.plafond.plafond_en_revenu_net_global * rbg_int, codev.plafond.plafond_par_personne)
        return min_(f6eh, plafond)


class grosses_reparations(Variable):
    value_type = float
    entity = FoyerFiscal
    label = 'Dépenses de grosses réparations des nus-propriétaires'
    reference = 'http://bofip.impots.gouv.fr/bofip/1852-PGP'
    definition_period = YEAR

    def formula_2009(foyer_fiscal, period, parameters):
        '''
        Dépenses de grosses réparations des nus-propriétaires (case 6CB et 6HJ)
        '''

        f6cb = foyer_fiscal('f6cb', period)
        plafond_grosses_reparations = parameters(period).impot_revenu.calcul_revenus_imposables.charges_deductibles.grosses_reparations.plafond

        return min_(f6cb, plafond_grosses_reparations)

    def formula_2010(foyer_fiscal, period, parameters):
        '''
        Dépenses de grosses réparations des nus-propriétaires
        '''
        f6cb = foyer_fiscal('f6cb', period)
        year = period.start.year

        report_depenses_depuis_2009 = sum(
            foyer_fiscal(case_report, period)
            for case_report in [
                'f6hj',
                'f6hk',
                'f6hl',
                'f6hm',
                'f6hn',
                'f6ho',
                'f6hp',
                'f6hq',
                'f6hr',
                ][0:year - 2009]
            )

        plafond_grosses_reparations = parameters(period).impot_revenu.calcul_revenus_imposables.charges_deductibles.grosses_reparations.plafond
        depenses_courantes = f6cb

        return min_(depenses_courantes + report_depenses_depuis_2009, plafond_grosses_reparations)

    def formula_2018_01_01(foyer_fiscal, period, parameters):
        '''
        Dépenses de grosses réparations des nus-propriétaires
        A partir de 2018: pas de nouveaux dépenses, excl. reports des années précedentes
        '''

        report_depenses_depuis_2009 = sum(
            foyer_fiscal(case_report, period)
            for case_report in [
                'f6hj',
                'f6hk',
                'f6hl',
                'f6hm',
                'f6hn',
                'f6ho',
                'f6hp',
                'f6hq',
                'f6hr',
                ]
            )

        plafond_grosses_reparations = parameters('2017-01-01').impot_revenu.calcul_revenus_imposables.charges_deductibles.grosses_reparations.plafond

        return min_(report_depenses_depuis_2009, plafond_grosses_reparations)

    def formula_2020_01_01(foyer_fiscal, period, parameters):
        '''
        Dépenses de grosses réparations des nus-propriétaires
        Depuis 2020: Reports à partir de 2010
        '''

        report_depenses_depuis_2010 = sum(
            foyer_fiscal(case_report, period)
            for case_report in [
                'f6hk',
                'f6hl',
                'f6hm',
                'f6hn',
                'f6ho',
                'f6hp',
                'f6hq',
                'f6hr',
                ]
            )

        plafond_grosses_reparations = parameters('2017-01-01').impot_revenu.calcul_revenus_imposables.charges_deductibles.grosses_reparations.plafond

        return min_(report_depenses_depuis_2010, plafond_grosses_reparations)



import os

from ..model.base import *


dir_path = os.path.join(os.path.dirname(__file__), 'parameters')


def modify_parameters(parameters):
    file_path = os.path.join(dir_path, 'allocations_familiales_imposables.yaml')
    reform_parameters_subtree = load_parameter_file(name='allocations_familiales_imposables', file_path=file_path)
    parameters.add_child('allocations_familiales_imposables', reform_parameters_subtree)
    return parameters


class allocations_familiales_imposables(Reform):
    name = 'Allocations familiales imposables'

    class rbg(Variable):
        label = 'Nouveau revenu brut global intégrant les allocations familiales'
        definition_period = YEAR

        def formula(foyer_fiscal, period, parameters):
            allocations_familiales_imposables = foyer_fiscal('allocations_familiales_imposables', period, options = [ADD])
            deficit_ante = foyer_fiscal('deficit_ante', period)
            f6gh = foyer_fiscal('f6gh', period)

            nacc_pvce_i = foyer_fiscal.members('nacc_pvce', period)
            nbic_impm_i = foyer_fiscal.members('nbic_impm', period)

            revenu_categoriel = foyer_fiscal('revenu_categoriel', period)
            cga = parameters(period).impot_revenu.calcul_revenus_imposables.rpns.cga_taux2

            nacc_pvce = foyer_fiscal.sum(nacc_pvce_i)
            return max_(
                0,
                allocations_familiales_imposables
                + revenu_categoriel
                + f6gh
                + (foyer_fiscal.sum(nbic_impm_i) + nacc_pvce)
                * (1 + cga)
                - deficit_ante
                )

    class rfr(Variable):
        label = 'Nouveau revenu fiscal de référence intégrant les allocations familiales'
        definition_period = YEAR

        def formula(foyer_fiscal, period, parameters):
            allocations_familiales_imposables = foyer_fiscal('allocations_familiales_imposables')
            abattements_plus_values = foyer_fiscal('abattements_plus_values')
            f3vi_holder = foyer_fiscal.members('f3vi')
            f3vz = foyer_fiscal('f3vz')
            rfr_cd = foyer_fiscal('rfr_cd')
            rni = foyer_fiscal('rni')
            rpns_exon_holder = foyer_fiscal.members('rpns_exon')
            rfr_rvcm_abattements_a_reintegrer = foyer_fiscal('rfr_rvcm_abattements_a_reintegrer')  # Supprimée en 2018
            revenus_capitaux_prelevement_liberatoire = foyer_fiscal('revenus_capitaux_prelevement_liberatoire', period, options = [ADD])  # Supprimée en 2018
            revenus_capitaux_prelevement_forfaitaire_unique_ir = foyer_fiscal('revenus_capitaux_prelevement_forfaitaire_unique_ir', period, options = [ADD])  # Existe à partir de 2018
            microentreprise = foyer_fiscal('microentreprise')

            f3vi = foyer_fiscal.sum(f3vi_holder)
            rpns_exon = foyer_fiscal.sum(rpns_exon_holder)

            return (
                max_(0, rni - allocations_familiales_imposables)
                + rfr_cd
                + rfr_rvcm_abattements_a_reintegrer
                + revenus_capitaux_prelevement_liberatoire
                + revenus_capitaux_prelevement_forfaitaire_unique_ir
                + f3vi
                + rpns_exon
                + abattements_plus_values
                + f3vz
                + microentreprise
                )

            # TO CHECK : f3vb after 2015 (abattements sur moins-values = interdits)

    class allocations_familiales_imposables(Variable):
        value_type = float
        entity = FoyerFiscal
        label = 'Allocations familiales imposables'
        definition_period = YEAR

        def formula(foyer_fiscal, period, parameters):
            imposition = parameters(period).allocations_familiales_imposables.imposition
            af = foyer_fiscal.declarant_principal.famille('af', period, options = [ADD])

            return af * imposition

    def apply(self):
        self.update_variable(self.rbg)
        self.update_variable(self.rfr)
        self.add_variable(self.allocations_familiales_imposables)
        self.modify_parameters(modifier_function = modify_parameters)



from openfisca_france.model.base import *
from openfisca_france.model.prelevements_obligatoires.prelevements_sociaux.cotisations_sociales.base import apply_bareme_for_relevant_type_sal


class stage_duree_heures(Variable):
    value_type = int
    entity = Individu
    label = "Nombre d'heures effectuées en stage"
    definition_period = MONTH
    set_input = set_input_divide_by_period


class stage_gratification_taux(Variable):
    value_type = float
    entity = Individu
    label = 'Taux de gratification (en plafond de la Sécurité sociale)'
    definition_period = MONTH
    set_input = set_input_dispatch_by_period


class stage_gratification(Variable):
    value_type = float
    entity = Individu
    label = 'Gratification de stage'
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula_2014_11(individu, period, parameters):
        stage_duree_heures = individu('stage_duree_heures', period)
        stage_gratification_taux = individu('stage_gratification_taux', period)
        stagiaire = individu('stagiaire', period)
        plafond_securite_sociale_horaire = parameters(period).prelevements_sociaux.pss.plafond_securite_sociale_horaire
        stage_gratification_taux_min = parameters(period).marche_travail.salaire_minimum.minstage.taux_gratification_min
        return stagiaire * plafond_securite_sociale_horaire * stage_duree_heures * max_(
            stage_gratification_taux, stage_gratification_taux_min)


class stage_gratification_reintegration(Variable):
    value_type = float
    entity = Individu
    label = "Part de la gratification de stage réintégrée à l'assiette des cotisations et contributions sociales"
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula_2014_11(individu, period, parameters):
        stage_duree_heures = individu('stage_duree_heures', period)
        stage_gratification = individu('stage_gratification', period)
        plafond_securite_sociale_horaire = parameters(period).prelevements_sociaux.pss.plafond_securite_sociale_horaire
        stage_gratification_taux_min = parameters(period).marche_travail.salaire_minimum.minstage.taux_gratification_min
        stage_gratification_min = plafond_securite_sociale_horaire * stage_duree_heures * stage_gratification_taux_min
        return max_(stage_gratification - stage_gratification_min, 0)


class stagiaire(Variable):
    value_type = bool
    entity = Individu
    label = "L'individu est stagiaire"
    definition_period = MONTH
    set_input = set_input_dispatch_by_period

    def formula(individu, period, parameters):
        stage_duree_heures = individu('stage_duree_heures', period)
        return (stage_duree_heures > 0)


class exoneration_cotisations_employeur_stagiaire(Variable):
    value_type = float
    entity = Individu
    label = "Exonération de cotisations employeur sur gratification d'un stagaire"
    reference = 'https://www.urssaf.fr/portail/home/employeur/calculer-les-cotisations/la-base-de-calcul/cas-particuliers--bases-forfaita/le-stagiaire-en-milieu-professio/la-franchise-de-cotisations-et-c.html'
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(individu, period, parameters):
        agirc_arrco_employeur = individu('agirc_arrco_employeur', period)
        agirc_employeur = individu('agirc_employeur', period)
        agirc_gmp_employeur = individu('agirc_gmp_employeur', period)
        arrco_employeur = individu('arrco_employeur', period)
        contribution_equilibre_general_employeur = individu('contribution_equilibre_general_employeur', period)
        contribution_equilibre_technique_employeur = individu('contribution_equilibre_technique_employeur', period)
        cotisation_exceptionnelle_temporaire_employeur = individu('cotisation_exceptionnelle_temporaire_employeur', period)
        plafond_securite_sociale = individu('plafond_securite_sociale', period)
        stage_gratification_reintegration = individu('stage_gratification_reintegration', period)
        stagiaire = individu('stagiaire', period)
        categorie_salarie = individu('categorie_salarie', period)

        bareme_by_type_sal_name = parameters(period).cotsoc.cotisations_employeur
        exoneration = sum(
            apply_bareme_for_relevant_type_sal(
                bareme_by_type_sal_name = bareme_by_type_sal_name,
                bareme_name = bareme_name,
                categorie_salarie = categorie_salarie,
                base = stage_gratification_reintegration,
                plafond_securite_sociale = plafond_securite_sociale,
                round_base_decimals = 2,
                )
            for bareme_name in ['agffnc', 'agffc', 'ags', 'chomage', 'asf']
            )
        exoneration += (agirc_arrco_employeur + agirc_employeur + agirc_gmp_employeur + arrco_employeur
            + contribution_equilibre_general_employeur + contribution_equilibre_technique_employeur
            + cotisation_exceptionnelle_temporaire_employeur)

        return - exoneration * stagiaire


class exoneration_cotisations_salarie_stagiaire(Variable):
    value_type = float
    entity = Individu
    label = 'Exonrérations de cotisations salarié pour un stagiaire'
    reference = 'http://www.apce.com/pid2798/stages.html?espace=3'
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(individu, period, parameters):
        agirc_arrco_salarie = individu('agirc_arrco_salarie', period)
        agirc_salarie = individu('agirc_salarie', period)
        agirc_gmp_salarie = individu('agirc_gmp_salarie', period)
        arrco_salarie = individu('arrco_salarie', period)
        contribution_equilibre_general_salarie = individu('contribution_equilibre_general_salarie', period)
        contribution_equilibre_technique_salarie = individu('contribution_equilibre_technique_salarie', period)
        cotisation_exceptionnelle_temporaire_salarie = individu('cotisation_exceptionnelle_temporaire_salarie', period)
        plafond_securite_sociale = individu('plafond_securite_sociale', period)
        stage_gratification_reintegration = individu('stage_gratification_reintegration', period)
        stagiaire = individu('stagiaire', period)
        categorie_salarie = individu('categorie_salarie', period)

        bareme_by_type_sal_name = parameters(period).cotsoc.cotisations_salarie
        bareme_names = ['agff', 'chomage', 'asf']

        exoneration = plafond_securite_sociale * 0.0
        for bareme_name in bareme_names:
            exoneration += apply_bareme_for_relevant_type_sal(
                bareme_by_type_sal_name = bareme_by_type_sal_name,
                bareme_name = bareme_name,
                categorie_salarie = categorie_salarie,
                base = stage_gratification_reintegration,
                plafond_securite_sociale = plafond_securite_sociale,
                round_base_decimals = 2,
                )
        exoneration = (exoneration + agirc_salarie + agirc_gmp_salarie + arrco_salarie + agirc_arrco_salarie
            + contribution_equilibre_general_salarie + contribution_equilibre_technique_salarie
            + cotisation_exceptionnelle_temporaire_salarie)

        return - exoneration * stagiaire



from numpy import datetime64
from openfisca_france.model.base import (
    Variable, Individu,
    MONTH, ADD, set_input_divide_by_period,
    not_, where,
    TypesNiveauDiplome, TypesStatutOccupationLogement
    )


class aide_jeunes_diplomes_anciens_boursiers_base_ressources(Variable):
    value_type = float
    entity = Individu
    reference = [
        "Article 3 du décret n° 2020-1789 du 30 décembre 2020 instituant une aide financière à titre exceptionnel à destination des jeunes diplômés en recherche d'emploi anciennement boursiers de l'enseignement supérieur",
        'https://www.legifrance.gouv.fr/eli/decret/2020/12/30/2020-1789/jo/article_3'
        ]
    label = "Ressources mensuelles de référence de l'aide perçue au cours de la dernière année d'étude"
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(individu, period, parameters):
        bourse_2020 = individu('bourse_criteres_sociaux', 2020, options = [ADD])
        bourse_2021 = individu('bourse_criteres_sociaux', 2021, options = [ADD])
        nombre_mensualites = parameters(period).prestations_sociales.aides_jeunes.bourses.bourses_enseignement_superieur.criteres_sociaux.nombre_mensualites

        return where(
            bourse_2021 > 0,
            bourse_2021 / nombre_mensualites,
            bourse_2020 / nombre_mensualites
            )


class aide_jeunes_diplomes_anciens_boursiers_eligibilite(Variable):
    value_type = bool
    entity = Individu
    label = "Éligibilité à l'aide jeunes diplômés et anciens boursiers"
    definition_period = MONTH
    end = '2021-12-31'
    reference = 'https://www.pole-emploi.fr/candidat/mes-droits-aux-aides-et-allocati/allocations-et-aides--les-repons/aides-financieres-aux-jeunes-dip.html'
    documentation = '''
    Conditions non modélisées :
    Être disponible pour occuper un emploi, le jour de la demande.
    Ne pas être inscrit dans une nouvelle formation (de niveau 5 minimum) dans l'année universitaire qui suit l'obtention du diplôme.
    '''

    def formula_2021_02_05(individu, period, parameters):
        age_limite = parameters(period).prestations_sociales.solidarite_insertion.autre_solidarite.covid19.aide_jeunes_diplomes_anciens_boursiers.age_limite
        condition_age = individu('age', period) < age_limite

        niveau_diplome = individu('plus_haut_diplome_niveau', period)
        date_diplome = individu('plus_haut_diplome_date_obtention', period)

        condition_diplome = (
            (niveau_diplome == TypesNiveauDiplome.niveau_5)
            + (niveau_diplome == TypesNiveauDiplome.niveau_6)
            + (niveau_diplome == TypesNiveauDiplome.niveau_7)
            + (niveau_diplome == TypesNiveauDiplome.niveau_8)
            ) * (datetime64('2020') <= date_diplome) * (date_diplome < datetime64('2022'))

        # bourse au cours de la dernière année de préparation du diplôme
        condition_bourse = individu('aide_jeunes_diplomes_anciens_boursiers_base_ressources', period) > 0

        # ne pas être en formation (de niveau 5 minimum) au moment de la demande.
        pas_en_formation = individu('niveau_diplome_formation', period) == TypesNiveauDiplome.non_renseigne

        # décale le test de non-cumul avec d'autres ressources d'un mois dans le passé
        last_month = period.last_month
        condition_non_cumul = not_(
            individu('chomage_brut', last_month)
            + individu('ass', last_month)
            + individu('allocation_travailleur_independant', last_month)
            + individu.famille('rsa', last_month)
            + individu('garantie_jeunes', last_month)
            )

        return condition_age * condition_diplome * condition_bourse * pas_en_formation * condition_non_cumul


class aide_jeunes_diplomes_anciens_boursiers_montant(Variable):
    value_type = float
    entity = Individu
    label = "Montant de l'aide jeunes diplômés et anciens boursiers"
    definition_period = MONTH
    end = '2021-12-31'
    reference = 'https://www.pole-emploi.fr/candidat/mes-droits-aux-aides-et-allocati/allocations-et-aides--les-repons/aides-financieres-aux-jeunes-dip.html'

    def formula_2021_02_05(individu, period, parameters):
        aide_jeunes_diplomes_anciens_boursiers_eligibilite = individu('aide_jeunes_diplomes_anciens_boursiers_eligibilite', period)

        bourse_precedente = individu('aide_jeunes_diplomes_anciens_boursiers_base_ressources', period)
        parameters_aide = parameters(period).prestations_sociales.solidarite_insertion.autre_solidarite.covid19.aide_jeunes_diplomes_anciens_boursiers

        part_bourse = bourse_precedente * parameters_aide.taux_bourse

        statut_occupation_logement = individu.menage('statut_occupation_logement', period)
        condition_logement = not_(
            (statut_occupation_logement == TypesStatutOccupationLogement.loge_gratuitement)
            + (statut_occupation_logement == TypesStatutOccupationLogement.non_renseigne)
            )
        part_logement = condition_logement * parameters_aide.majoration_logement

        return aide_jeunes_diplomes_anciens_boursiers_eligibilite * (part_bourse + part_logement)



import logging
from openfisca_france.model.base import *

log = logging.getLogger(__name__)

# Variables utilisées à la fois pour le PFL et pour la partie IR du PFU


class produit_epargne_solidaire(Variable):
    value_type = float
    entity = FoyerFiscal
    label = "Produit d'épargne solidaire"
    definition_period = YEAR


class produit_etats_non_cooperatif(Variable):
    value_type = float
    entity = FoyerFiscal
    label = "Produits de placement à revenus fixe ou de contrats de capitalisation et d'assurance-vie versés à un bénéficiaire résidant dans un état non-coopératif"
    definition_period = YEAR



#! /usr/bin/env python
'''
Measure performance of formula calculations to compare to other OpenFisca implementations.

Usage example:

    python openfisca_france/scripts/performance/measure_calculations_performance.py
'''


import argparse
import logging
import sys
import time

import numpy as np
from openfisca_core import periods, simulations
from openfisca_core.tools import assert_near
from openfisca_france import FranceTaxBenefitSystem


args = None
logger = logging.getLogger(__name__)


def add_member(entity, **variables_value_by_name):
    entity.count += 1
    entity.step_size += 1
    member_index = entity.count - 1
    simulation = entity.simulation

    # Add a cell to all arrays of all variables of entity.
    for _variable_name, variable_holder in entity.holder_by_name.items():
        column = variable_holder.variable
        if column.definition_period is ETERNITY:   # noqa F821
            variable_holder._array = np.hstack((variable_holder._array, [column.default_value]))
        else:
            array_by_period = variable_holder._array_by_period
            if array_by_period is None:
                variable_holder._array_by_period = array_by_period = {}
            for period, array in array_by_period.items():
                array_by_period[period] = np.hstack((array, [column.default_value]))

    # When entity is a person, ensure that the index & role of the person in the other entities are set.
    value_by_name = variables_value_by_name.copy()
    if entity.is_persons_entity:
        for other_entity in iter(simulation.entity_by_key_singular.values()):
            if not other_entity.is_persons_entity:
                assert other_entity.count > 0
                value_by_name.setdefault(other_entity.index_for_person_variable_name, other_entity.count - 1)
                role = value_by_name.get(other_entity.role_for_person_variable_name)
                assert role is not None, 'Missing role {} in person arguments: {}'.format(
                    other_entity.role_for_person_variable_name, value_by_name)
                if role >= other_entity.roles_count:
                    other_entity.roles_count = role + 1

    # Set arguments in variables.
    for variable_name, value in value_by_name.items():
        variable_holder = simulation.get_or_new_holder(variable_name)
        column = variable_holder.variable
        if isinstance(value, dict):
            for period, period_value in value.items():
                array = variable_holder.get_array(period)
                if array is None:
                    array = np.empty(entity.count, dtype = column.dtype)
                    array.fill(column.default_value)
                    variable_holder.put_in_cache(array, period)
                array[member_index] = period_value
        else:
            period = simulation.period
            array = variable_holder.get_array(period)
            if array is None:
                array = np.empty(entity.count, dtype = column.dtype)
                array.fill(column.default_value)
                variable_holder.put_in_cache(array, period)
            array[member_index] = value

    return member_index


def timeit(method):
    def timed(*args, **kwargs):
        start_time = time.time()
        result = method(*args, **kwargs)
        # logger.debug '%r (%r, %r) %2.9f s' % (method.__name__, args, kw, time.time() - start_time)
        logger.debug('{:2.6f} s'.format(time.time() - start_time))
        return result

    return timed


tax_benefit_system = FranceTaxBenefitSystem()


@timeit
def test_irpp(year, irpp, **variables_value_by_name):
    simulation = simulations.Simulation(period = periods.period(year), tax_benefit_system = tax_benefit_system,
        debug = args.verbose)
    famille = simulation.entity_by_key_singular['famille']
    foyer_fiscal = simulation.entity_by_key_singular['foyer_fiscal']
    individu = simulation.entity_by_key_singular['individu']
    menage = simulation.entity_by_key_singular['menage']

    # Dispatch arguments to their respective entities.
    variables_value_by_name_by_entity = {}
    for variable_name, value in variables_value_by_name.items():
        variable_holder = simulation.get_or_new_holder(variable_name)
        entity_variables_value_by_name = variables_value_by_name_by_entity.setdefault(variable_holder.entity, {})
        entity_variables_value_by_name[variable_name] = value

    add_member(famille, **variables_value_by_name_by_entity.get(famille, {}))
    add_member(foyer_fiscal, **variables_value_by_name_by_entity.get(foyer_fiscal, {}))
    add_member(menage, **variables_value_by_name_by_entity.get(menage, {}))
    add_member(individu, quifam = 1, quifoy = 1, quimen = 1, **variables_value_by_name_by_entity.get(individu, {}))
    assert_near(simulation.calculate('irpp'), irpp, absolute_error_margin = 0.51)


def main():
    parser = argparse.ArgumentParser(description = __doc__)
    parser.add_argument('-v', '--verbose', action = 'store_true', default = False, help = 'increase output verbosity')
    global args
    args = parser.parse_args()
    logging.basicConfig(level = logging.DEBUG if args.verbose else logging.WARNING, stream = sys.stdout)

    logger.debug('salaire_imposable')

    test_irpp(2010, -1181, salaire_imposable = 20000)
    test_irpp(2010, -7934, salaire_imposable = 50000)
    test_irpp(2010, -42338, salaire_imposable = 150000)
    test_irpp(2011, -1181, salaire_imposable = 20000)
    test_irpp(2011, -7934, salaire_imposable = 50000)
    test_irpp(2011, -42338, salaire_imposable = 150000)
    test_irpp(2012, -1181, salaire_imposable = 20000)
    test_irpp(2012, -7934, salaire_imposable = 50000)
    test_irpp(2012, -43222, salaire_imposable = 150000)
    test_irpp(2013, -1170, salaire_imposable = 20000)
    test_irpp(2013, -7889, salaire_imposable = 50000)
    test_irpp(2013, -43076, salaire_imposable = 150000)

    logger.debug('retraite_imposable')

    test_irpp(2010, -1181, retraite_imposable = 20000)
    test_irpp(2010, -8336, retraite_imposable = 50000)
    test_irpp(2010, -46642, retraite_imposable = 150000)
    test_irpp(2011, -1181, retraite_imposable = 20000)
    test_irpp(2011, -8336, retraite_imposable = 50000)
    test_irpp(2011, -46642, retraite_imposable = 150000)
    test_irpp(2012, -1181, retraite_imposable = 20000)
    test_irpp(2012, -8336, retraite_imposable = 50000)
    test_irpp(2012, -46642, retraite_imposable = 150000)
    test_irpp(2013, -1170, retraite_imposable = 20000)
    test_irpp(2013, -8283, retraite_imposable = 50000)
    test_irpp(2013, -46523, retraite_imposable = 150000)

    logger.debug('f2da')

    test_irpp(2010, 0, f2da = 20000)
    test_irpp(2010, 0, f2da = 50000)
    test_irpp(2010, 0, f2da = 150000)
    test_irpp(2011, 0, f2da = 20000)
    test_irpp(2011, 0, f2da = 50000)
    test_irpp(2011, 0, f2da = 150000)
    test_irpp(2012, 0, f2da = 20000)
    test_irpp(2012, 0, f2da = 50000)
    test_irpp(2012, 0, f2da = 150000)
    # test_irpp(2013, 0, f2da = 20000)
    # test_irpp(2013, 0, f2da = 50000)
    # test_irpp(2013, 0, f2da = 150000)

    logger.debug('f2dc')

    test_irpp(2010, 0, f2dc = 20000)
    test_irpp(2010, -2976, f2dc = 50000)
    test_irpp(2010, -22917, f2dc = 150000)
    test_irpp(2011, 0, f2dc = 20000)
    test_irpp(2011, -2976, f2dc = 50000)
    test_irpp(2011, -22917, f2dc = 150000)
    test_irpp(2012, 0, f2dc = 20000)
    test_irpp(2012, -3434, f2dc = 50000)
    test_irpp(2012, -23542, f2dc = 150000)
    # test_irpp(2013, 0, f2dc = 20000)
    # test_irpp(2013, 0, f2dc = 50000)
    # test_irpp(2013, 0, f2dc = 150000)

    logger.debug('f2dh')

    test_irpp(2010, 345, f2dh = 20000)
    test_irpp(2010, 345, f2dh = 50000)
    test_irpp(2010, 345, f2dh = 150000)
    test_irpp(2011, 345, f2dh = 20000)
    test_irpp(2011, 345, f2dh = 50000)
    test_irpp(2011, 345, f2dh = 150000)
    test_irpp(2012, 345, f2dh = 20000)
    test_irpp(2012, 345, f2dh = 50000)
    test_irpp(2012, 345, f2dh = 150000)
    test_irpp(2013, 345, f2dh = 20000)
    test_irpp(2013, 345, f2dh = 50000)
    test_irpp(2013, 345, f2dh = 150000)

    logger.debug('f2tr')

    test_irpp(2010, -1461, f2tr = 20000)
    test_irpp(2010, -9434, f2tr = 50000)
    test_irpp(2010, -48142, f2tr = 150000)
    test_irpp(2011, -1461, f2tr = 20000)
    test_irpp(2011, -9434, f2tr = 50000)
    test_irpp(2011, -48142, f2tr = 150000)
    test_irpp(2012, -1461, f2tr = 20000)
    test_irpp(2012, -9434, f2tr = 50000)
    test_irpp(2012, -48142, f2tr = 150000)
    test_irpp(2013, -1450, f2tr = 20000)
    test_irpp(2013, -9389, f2tr = 50000)
    test_irpp(2013, -48036, f2tr = 150000)

    logger.debug('f2ts')

    test_irpp(2010, -1461, f2ts = 20000)
    test_irpp(2010, -9434, f2ts = 50000)
    test_irpp(2010, -48142, f2ts = 150000)
    test_irpp(2011, -1461, f2ts = 20000)
    test_irpp(2011, -9434, f2ts = 50000)
    test_irpp(2011, -48142, f2ts = 150000)
    test_irpp(2012, -1461, f2ts = 20000)
    test_irpp(2012, -9434, f2ts = 50000)
    test_irpp(2012, -48142, f2ts = 150000)
    test_irpp(2013, -1450, f2ts = 20000)
    test_irpp(2013, -9389, f2ts = 50000)
    test_irpp(2013, -48036, f2ts = 150000)

    logger.debug('f3vg')

    test_irpp(2010, -3600, f3vg = 20000)
    test_irpp(2010, -9000, f3vg = 50000)
    test_irpp(2010, -27000, f3vg = 150000)
    test_irpp(2011, -3800, f3vg = 20000)
    test_irpp(2011, -9500, f3vg = 50000)
    test_irpp(2011, -28500, f3vg = 150000)
    test_irpp(2012, -4800, f3vg = 20000)
    test_irpp(2012, -12000, f3vg = 50000)
    test_irpp(2012, -36000, f3vg = 150000)
    test_irpp(2013, -1450, f3vg = 20000)
    test_irpp(2013, -9389, f3vg = 50000)
    test_irpp(2013, -48036, f3vg = 150000)

    logger.debug('f3vz')

    # test_irpp(2010, 0, f3vz = 20000)
    # test_irpp(2010, 0, f3vz = 50000)
    # test_irpp(2010, 0, f3vz = 150000)
    test_irpp(2011, 0, f3vz = 20000)
    test_irpp(2011, 0, f3vz = 50000)
    test_irpp(2011, 0, f3vz = 150000)
    test_irpp(2012, 0, f3vz = 20000)
    test_irpp(2012, 0, f3vz = 50000)
    test_irpp(2012, 0, f3vz = 150000)
    test_irpp(2013, 0, f3vz = 20000)
    test_irpp(2013, 0, f3vz = 50000)
    test_irpp(2013, 0, f3vz = 150000)

    logger.debug('f4ba')

    test_irpp(2010, -1461, f4ba = 20000)
    test_irpp(2010, -9434, f4ba = 50000)
    test_irpp(2010, -48142, f4ba = 150000)
    test_irpp(2011, -1461, f4ba = 20000)
    test_irpp(2011, -9434, f4ba = 50000)
    test_irpp(2011, -48142, f4ba = 150000)
    test_irpp(2012, -1461, f4ba = 20000)
    test_irpp(2012, -9434, f4ba = 50000)
    test_irpp(2012, -48142, f4ba = 150000)
    test_irpp(2013, -1450, f4ba = 20000)
    test_irpp(2013, -9389, f4ba = 50000)
    test_irpp(2013, -48036, f4ba = 150000)


if __name__ == '__main__':
    sys.exit(main())



''' xml_to_yaml_france.py : Parse XML parameter files for Openfisca-France and convert them to YAML files. Comments are NOT transformed.

Usage :
  `python xml_to_yaml_france.py output_dir`
or just (output is written in a directory called `yaml_parameters`):
  `python xml_to_yaml_france.py`
'''

import sys
import os

from openfisca_france.france_taxbenefitsystem import COUNTRY_DIR
from openfisca_core.scripts.migrations.v16_2_to_v17 import xml_to_yaml


if len(sys.argv) > 1:
    target_path = sys.argv[1]
else:
    target_path = 'yaml_parameters'

param_dir = os.path.join(COUNTRY_DIR, 'parameters')
param_files = [
    'bouclier_fiscal.xml',
    'bourses_education.xml',
    'cmu.xml',
    'cotsoc.xml',
    'fonc.xml',
    'impot_revenu.xml',
    'prelevements_sociaux.xml',
    'prestations_sociales.xml',
    'taxation_capital.xml',
    'tns.xml',
    ]
legislation_xml_info_list = [
    (os.path.join(param_dir, param_file), [])
    for param_file in param_files
    ]

xml_to_yaml.write_parameters(legislation_xml_info_list, target_path)



from openfisca_france.model.base import *

# PLAN :
# 1) Gains taxés comme des salaires
# 2) Autres plus-values (exonérées, au barème, ...)
# 3) Abattements sur plus-values
# 4) Plus-values taxées forfaitairement
# 5) Autres variables


# Gains taxés comme des salaires

class f1tt(Variable):
    cerfa_field = {
        0: '1TT',
        1: '1UT',
        }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = "Gains de levée d'options sur titres et gains d'acquisition d'actions gratuites attribuées à compter du 28.9.2012"
    # start_date = date(2012, 1, 1)
    definition_period = YEAR


class f1tv(Variable):
    cerfa_field = {
        0: '1TV',
        1: '1UV',
        }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = "Gains de levée d'options sur titres en cas de cession ou de conversion au porteur dans le délai d'indisponibilité: entre 1 et 2 ans"
    end = '2015-12-31'
    definition_period = YEAR


class f1tw(Variable):
    cerfa_field = {
        0: '1TW',
        1: '1UW',
        }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = "Gains de levée d'options sur titres en cas de cession ou de conversion au porteur dans le délai d'indisponibilité: entre 2 et 3 ans"
    end = '2015-12-31'
    definition_period = YEAR


class f1tx(Variable):
    cerfa_field = {
        0: '1TX',
        1: '1UX',
        }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = "Gains de levée d'options sur titres en cas de cession ou de conversion au porteur dans le délai d'indisponibilité: entre 3 et 4 ans"
    end = '2016-12-31'
    definition_period = YEAR


class f3vj(Variable):
    cerfa_field = {
        0: '3VJ',
        1: '3VK',
        }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Gains imposables sur option dans la catégorie des salaires'
    definition_period = YEAR


# Autres plus-values

class f3vg(Variable):
    cerfa_field = '3VG'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Plus-value imposable sur gains de cession de valeurs mobilières, de droits sociaux et gains assimilés'
    definition_period = YEAR


class f3vh(Variable):
    cerfa_field = '3VH'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Moins-value imposable su gains de cession de valeurs mobilières, de droits sociaux et gains assimilés'
    definition_period = YEAR


class f3vv(Variable):
    cerfa_field = '3VV'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Plus-values réalisées par les non-résidents: montant du prélèvement de 45 % déjà versé'
    # start_date = date(2013, 1, 1)
    end = '2013-12-31'
    definition_period = YEAR


class f3vz(Variable):
    cerfa_field = '3VZ'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Plus-values imposables sur cessions d’immeubles ou de biens meubles'
    # start_date = date(2011, 1, 1)
    definition_period = YEAR


class f3vc(Variable):
    cerfa_field = '3VC'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Produits et plus-values exonérés provenant de structure de capital-risque'
    # start_date = date(2006, 1, 1)
    definition_period = YEAR


class f3vp(Variable):
    cerfa_field = '3VP'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Plus-values exonérées de cessions de titres de jeunes entreprises innovantes'
    # start_date = date(2007, 1, 1)
    end = '2013-12-31'
    definition_period = YEAR


class f3vq(Variable):
    cerfa_field = '3VQ'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = "Cessions de titres détenus à l'étranger par les impatriés : plus-values exonérées (50%)"
    definition_period = YEAR


class f3vr(Variable):
    cerfa_field = '3VR'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = "Cessions de titres détenus à l'étranger par les impatriés : moins-values non imputables (50%)"
    definition_period = YEAR


class f3vy(Variable):
    cerfa_field = '3VY'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Plus-values exonérées de cessions de participations supérieures à 25% au sein du groupe familial'
    # start_date = date(2011, 1, 1)
    end = '2013-12-31'
    definition_period = YEAR


class f3ve(Variable):
    cerfa_field = '3VE'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = "Plus-values réalisées par les non-résidents pour lesquelles vous demandez le remboursement de l'excédent du prélèvement de 45 %"
    definition_period = YEAR


class f3tz(Variable):
    cerfa_field = '3TZ'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = "Plus-values de cession de titres d'OPC monétaires en report d'imposition, plus-values réalisées du 1.1 au 31.3.2017, plus-values en report d'imposition"
    # start_date = date(2016, 1, 1)
    end = '2017-12-31'
    definition_period = YEAR


class f3sa(Variable):
    cerfa_field = '3SA'
    value_type = int
    entity = FoyerFiscal
    label = "Plus-values en report d'imposition, dont le report a expiré cette année; montant avant abattement"
    # start_date = date(2016, 1, 1)
    definition_period = YEAR


class f3sb(Variable):
    cerfa_field = '3SB'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = "Plus-values en report d'imposition, dont le report a expiré cette année; montant imposable"
    # start_date = date(2013, 1, 1)
    definition_period = YEAR


class f3wb(Variable):
    cerfa_field = '3WB'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Plus-values des individus transférant leur domicile fiscal hors de France; plus-values et créances ne bénéficiant pas du sursis de paiement; plus-values nettes imposables au barème'
    # start_date = date(2013, 1, 1)
    definition_period = YEAR


class f3wd(Variable):
    cerfa_field = '3WD'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Plus-values des individus transférant leur domicile fiscal hors de France; plus-values et créances ne bénéficiant pas du sursis de paiement; plus-values de base soumise aux prélèvements sociaux'
    # start_date = date(2013, 1, 1)
    definition_period = YEAR


class f3we(Variable):
    '''
    Plus-values en report d'imposition au sens de l'art. 150-0 D bis du CGI :
    Jusqu'aux revenus de 2013 : montants nets réalisés pendant l'année
    Pour les revenus 2014 : complément net de prix perçu pendant l'année (car fin du dispositif)
    Depuis les revenus 2015 : complément brut de prix perçu pendant l'année (l'abattement n'est plus recensé dans les cases 3SG et 3SL)
    '''
    cerfa_field = '3WE'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = "Plus-values en report d'imposition au sens de l'art. 150-0 D bis du CGI"
    definition_period = YEAR


class f3wn(Variable):
    cerfa_field = '3WN'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = "Plus-values en report d'imposition dont le report a expiré cette année, réalisées à compter du 1.1.2013 : plus-values avant abattement"
    # start_date = date(2016, 1, 1)
    definition_period = YEAR


class f3wp(Variable):
    cerfa_field = '3WP'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = "Plus-values en report d'imposition dont le report a expiré cette année, réalisées à compter du 1.1.2013 : plus-values après abattement"
    # start_date = date(2016, 1, 1)
    definition_period = YEAR


class f3wr(Variable):
    cerfa_field = '3WR'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = "Plus-values en report d'imposition dont le report a expiré cette année, réalisées à compter du 1.1.2013 : impôt sur le revenu"
    # start_date = date(2016, 1, 1)
    definition_period = YEAR


class f3wt(Variable):
    cerfa_field = '3WT'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = "Plus-values en report d'imposition dont le report a expiré cette année, réalisées à compter du 1.1.2013 : contribution exceptionnelle sur les hauts revenus"
    # start_date = date(2016, 1, 1)
    definition_period = YEAR


# Abattements sur plus-values

class f3va_2014(Variable):
    cerfa_field = '3VA'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Abattements nets (abattement pour durée de détention renforcé et abattement fixe spécial) appliqués sur des plus-values réalisées par les dirigeants de PME lors de leur départ à la retraite'
    # start_date = date(2006, 1, 1)
    end = '2014-12-31'
    definition_period = YEAR


class f3va_2016(Variable):
    cerfa_field = {
        0: '3VA',
        1: '3VB',
        2: '3VO',
        3: '3VP',
        }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Abattements nets (abattement pour durée de détention renforcé et abattement fixe spécial) appliqués sur des plus-values réalisées par les dirigeants de PME lors de leur départ à la retraite'
    # start_date = date(2015, 1, 1)
    end = '2016-12-31'
    definition_period = YEAR


class f3va(Variable):
    cerfa_field = '3VA'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Abattement fixe spécial appliqué sur des plus-values réalisées par les dirigeants de PME lors de leur départ à la retraite'
    # start_date = date(2017, 1, 1)
    definition_period = YEAR


class f3vb(Variable):
    cerfa_field = '3VB'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Abattements nets (abattement pour durée de détention renforcé et abattement fixe spécial) appliqués sur des moins-values réalisées par les dirigeants de PME lors de leur départ à la retraite'
    # start_date = date(2006, 1, 1)
    end = '2014-12-31'
    definition_period = YEAR


class f3sg(Variable):
    cerfa_field = '3SG'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Abattement net pour durée de détention (détention de droit commun à partir de 2015) : appliqué sur des plus-values'
    # start_date = date(2013, 1, 1)
    definition_period = YEAR


class f3sh(Variable):
    cerfa_field = '3SH'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Abattement net pour durée de détention : appliqué sur des moins-values'
    # start_date = date(2013, 1, 1)
    end = '2014-12-31'
    definition_period = YEAR


class f3sl(Variable):
    cerfa_field = '3SL'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Abattement net pour durée de détention renforcée : appliqué sur des plus-values'
    # start_date = date(2013, 1, 1)
    definition_period = YEAR


class f3sm(Variable):
    cerfa_field = '3SM'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Abattement net pour durée de détention renforcée : appliqué sur des moins-values'
    # start_date = date(2013, 1, 1)
    end = '2014-12-31'
    definition_period = YEAR


class abattements_plus_values(Variable):
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    reference = 'http://bofip.impots.gouv.fr/bofip/9540-PGP'
    label = "Abattements sur plus-values notamment pour durée de détention de droit commun, renforcé, et abattement en cas de départ à la retraite d'un dirigeant de PME (abattement fixe et pour durée de détention)"
    definition_period = YEAR
    end = '2017-12-31'

    def formula_2013_01_01(foyer_fiscal, period):
        f3sg = foyer_fiscal('f3sg', period)
        f3sh = foyer_fiscal('f3sh', period)
        f3sl = foyer_fiscal('f3sl', period)
        f3sm = foyer_fiscal('f3sm', period)
        f3va = foyer_fiscal('f3va_2014', period)
        f3vb = foyer_fiscal('f3vb', period)

        return max_(0, f3sg - f3sh) + max_(0, f3sl - f3sm) + f3va - f3vb

    def formula_2015_01_01(foyer_fiscal, period):
        f3sg = foyer_fiscal('f3sg', period)
        f3sl = foyer_fiscal('f3sl', period)
        f3va_i = foyer_fiscal.members('f3va_2016', period)
        f3va = foyer_fiscal.sum(f3va_i)

        return f3sg + f3sl + f3va

    def formula_2017_01_01(foyer_fiscal, period):
        f3sg = foyer_fiscal('f3sg', period)
        f3sl = foyer_fiscal('f3sl', period)
        f3va = foyer_fiscal('f3va', period)

        return f3sg + f3sl + f3va


# Plus values et gains taxables à des taux forfaitaires

class f3vd(Variable):
    ''' ATTENTION : à partir des revenus 2015, la case 3SD est supprimée : seule la case 3VD reste et recense les montants à l'échelle du foyer fiscal. Avec le code actuel, le seul problème serait si la case 3SD était réutilisée un jour pour autre chose dans le formulaire, ce qui n'est pas le cas aujourd'hui '''
    cerfa_field = {
        0: '3VD',
        1: '3SD',
        }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = "Gains de levée d'options sur titres et gains d'acquisition d'actions taxables à 18 %"
    # start_date = date(2008, 1, 1)
    definition_period = YEAR


class f3vi(Variable):
    ''' ATTENTION : à partir des revenus 2015, la case 3SI est supprimée : seule la case 3VI reste et recense les montants à l'échelle du foyer fiscal. Avec le code actuel, le seul problème serait si la case 3SI était réutilisée un jour pour autre chose dans le formulaire, ce qui n'est pas le cas aujourd'hui '''
    cerfa_field = {
        0: '3VI',
        1: '3SI',
        }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = "Gains de levée d'options sur titres et gains d'acquisition d'actions taxables à 30 %"
    definition_period = YEAR


class f3vf(Variable):
    ''' ATTENTION : à partir des revenus 2015, la case 3SF est supprimée : seule la case 3VF reste et recense les montants à l'échelle du foyer fiscal. Avec le code actuel, le seul problème serait si la case 3SF était réutilisée un jour pour autre chose dans le formulaire, ce qui n'est pas le cas aujourd'hui '''
    cerfa_field = {
        0: '3VF',
        1: '3SF',
        }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = "Gains de levée d'options sur titres et gains d'acquisition d'actions taxables à 41 %"
    definition_period = YEAR


class f3vm(Variable):
    cerfa_field = '3VM'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = "Clôture du PEA avant l'expiration de la 2e année: gains taxables à 22.5 %"
    # start_date = date(2012, 1, 1)
    end = '2018-12-31'
    definition_period = YEAR


class f3vt(Variable):
    cerfa_field = '3VT'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Clôture du PEA  entre la 2e et la 5e année: gains taxables à 19 %'  # Depuis IR 2020 :Retrait ou rachat du PEA ou du PEA-PME avant expiration de la 5e année"
    # start_date = date(2012, 1, 1)
    definition_period = YEAR


class f3vl(Variable):
    cerfa_field = '3VL'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Distributions par des sociétés de capital-risque taxables à 19 %'
    end = '2013-12-31'
    definition_period = YEAR


class f3sa_2012(Variable):
    cerfa_field = '3SA'
    value_type = int
    entity = FoyerFiscal
    label = 'Plus-values de cessions de titres réalisées par un entrepreneur, taxables à 19%'
    # start_date = date(2012, 1, 1)
    end = '2012-12-31'
    definition_period = YEAR


class f3sj(Variable):
    cerfa_field = '3SJ'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = "Gains de cession de bons de souscription de parts de créateur d'entreprise : gains taxables à 19%"
    definition_period = YEAR


class f3tj(Variable):
    cerfa_field = '3TJ'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = "Gains de cession de bons de souscription de parts de créateur d'entreprise attribués après 2018 : gains taxables à 12.8%"
    # start_date = date(2018, 1, 1)
    definition_period = YEAR


class f3tk(Variable):
    cerfa_field = '3TK'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = "Abattement forfaitaire sur les gains de cession de bons de souscription de parts de créateur d'entreprise (départ dirigeant PME)"
    # start_date = date(2019, 1, 1)
    definition_period = YEAR


class f3sk(Variable):
    cerfa_field = '3SK'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = "Gains de cession de bons de souscription de parts de créateur d'entreprise : gains taxables à 30%"
    definition_period = YEAR


class f3wi(Variable):
    cerfa_field = '3WI'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = "Plus-values en report d'imposition dont le report a expiré cette année : réalisées du 14.11.2012 au 31.12.2012 et taxables à 24%"
    # start_date = date(2016, 1, 1)
    definition_period = YEAR


class f3wj(Variable):
    cerfa_field = '3WJ'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = "Plus-values en report d'imposition dont le report a expiré cette année : réalisées du 14.11.2012 au 31.12.2012 et taxables à 19%"
    # start_date = date(2016, 1, 1)
    definition_period = YEAR


class f3an(Variable):
    cerfa_field = '3AN'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = "Plus-value réalisée sur la cession d'actifs numériques"
    # start_date = date(2019, 1, 1)
    definition_period = YEAR


class f3bn(Variable):
    cerfa_field = '3BN'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = "Moins-value réalisée sur la cession d'actifs numériques"
    # start_date = date(2019, 1, 1)
    definition_period = YEAR


class f3pi(Variable):
    cerfa_field = '3PI'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Profits sur instruments financiers réalisés dans des ETNC ou paradis fiscaux'
    # start_date = date(2017, 1, 1)
    definition_period = YEAR


# Autres variables

class f3vv_end_2010(Variable):
    cerfa_field = '3VV'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Pertes ouvrant droit au crédit d’impôt de 19 % '
    # start_date = date(2010, 1, 1)
    end = '2010-12-31'
    definition_period = YEAR


class f3ua(Variable):
    cerfa_field = '3UA'
    label = "Plus-values bénéficiant de l'abattement pour durée de détention renforcé et plus-values réalisées par les dirigeants de PME lors de leur départ à la retraite : plus-values après abattements"
    value_type = float
    entity = FoyerFiscal
    # start_date = date(2017, 1, 1) NB : Cette case existait avant 2017, mais les montants qui y étaient indiqués étaient également indiqués case 3VG
    definition_period = YEAR



import copy
import logging

from openfisca_france.model.base import *
from openfisca_france.model.revenus.activite.salarie import TypesCategorieSalarie


log = logging.getLogger(__name__)


def build_pat(node_json):  # Ici node_json c'est le dossier 'parameters'
    '''Construit le dictionnaire de barèmes des cotisations employeur à partir des paramètres de parameters.'''
    # TODO: contribution patronale de prévoyance complémentaire
    pat = ParameterNode('pat', data=dict(description='Cotisations sociales employeur'))  # Génère pat
    commun = ParameterNode('commun', data=dict(
        description='Cotisations sociales employeur communes à plusieurs régimes',
        metadata=dict(order=[]),
        ))  # Génère commun

    # Réindexation : nouveaux chemins suite à l'harmonisation avec les répertoires des barèmes IPP
    autres = node_json.prelevements_sociaux.autres_taxes_participations_assises_salaires
    retraites = node_json.prelevements_sociaux.regimes_complementaires_retraite_secteur_prive
    chomage = node_json.prelevements_sociaux.cotisations_regime_assurance_chomage
    regime_general = node_json.prelevements_sociaux.cotisations_securite_sociale_regime_general
    public = node_json.prelevements_sociaux.cotisations_secteur_public

    # Création de commun
    # Apprentissage (avec effacement)
    commun.children.update(autres.apprentissage.children)
    if autres.apprentissage.metadata is not None and autres.apprentissage.metadata.get('order') is not None:
        commun.metadata['order'] += autres.apprentissage.metadata['order']

    del commun.children['csa']  # n'est pas un barème et est utilisé directement
    del commun.children['cotisation_supplementaire']

    # Formation
    commun.children.update(autres.formation.employeur_tout_salaire.children)
    if autres.formation.employeur_tout_salaire.metadata is not None and autres.formation.employeur_tout_salaire.metadata.get('order') is not None:
        commun.metadata['order'] += autres.formation.employeur_tout_salaire.metadata['order']

    # Construction (avec renommage et effacement)
    commun.children.update(autres.construction.children)
    commun.children['construction_plus_de_10_salaries'] = commun.children.pop('10_19_salaries')
    commun.children['construction_plus_de_20_salaries'] = commun.children.pop('plus_20_salaries')
    commun.children['construction_plus_de_50_salaries'] = commun.children.pop('plus_50_salaries')
    del commun.children['seuil']
    commun.metadata['order'] += [
        'construction_plus_de_10_salaries',
        'construction_plus_de_20_salaries',
        'construction_plus_de_50_salaries',
        ]
    # Autres thématiques
    commun.children.update(chomage.ags.employeur.children)
    if chomage.ags.employeur.metadata is not None and chomage.ags.employeur.metadata.get('order') is not None:
        commun.metadata['order'] += chomage.ags.employeur.metadata['order']
    commun.children.update(chomage.asf.employeur.children)
    if chomage.asf.employeur.metadata is not None and chomage.asf.employeur.metadata.get('order') is not None:
        commun.metadata['order'] += chomage.asf.employeur.metadata['order']
    commun.children.update(chomage.chomage.employeur.children)
    if chomage.chomage.employeur.metadata is not None and chomage.chomage.employeur.metadata.get('order') is not None:
        commun.metadata['order'] += chomage.chomage.employeur.metadata['order']
    commun.children.update(regime_general.csa.employeur.children)
    if regime_general.csa.employeur.metadata is not None and regime_general.csa.employeur.metadata.get('order') is not None:
        commun.metadata['order'] += regime_general.csa.employeur.metadata['order']
    commun.children.update(regime_general.famille.employeur.children)
    if regime_general.famille.employeur.metadata is not None and regime_general.famille.employeur.metadata.get('order') is not None:
        commun.metadata['order'] += regime_general.famille.employeur.metadata['order']
    commun.children.update(regime_general.penibilite.children)
    if regime_general.penibilite.metadata is not None and regime_general.penibilite.metadata.get('order') is not None:
        commun.metadata['order'] += regime_general.penibilite.metadata['order']
    commun.children.update(regime_general.cnav.employeur.children)
    if regime_general.cnav.employeur.metadata is not None and regime_general.cnav.employeur.metadata.get('order') is not None:
        commun.metadata['order'] += regime_general.cnav.employeur.metadata['order']
    commun.children.update(regime_general.mmid.employeur.children)
    if regime_general.mmid.employeur.metadata is not None and regime_general.mmid.employeur.metadata.get('order') is not None:
        commun.metadata['order'] += regime_general.mmid.employeur.metadata['order']

    # Fnal (avec renommage)
    commun.children.update(autres.fnal.children)
    commun.children['fnal_contribution_moins_de_20_salaries'] = commun.children.pop('contribution_moins_de_20_salaries')
    commun.children['fnal_contribution_moins_de_50_salaries'] = commun.children.pop('contribution_moins_de_50_salaries')
    commun.children['fnal_contribution_plus_de_10_salaries'] = commun.children.pop('contribution_plus_de_10_salaries')
    commun.children['fnal_contribution_plus_de_20_salaries'] = commun.children.pop('contribution_plus_de_20_salaries')
    commun.children['fnal_contribution_plus_de_50_salaries'] = commun.children.pop('contribution_plus_de_50_salaries')
    commun.children['fnal_cotisation'] = commun.children.pop('cotisation')
    del commun.children['tout_employeur']
    commun.metadata['order'] += [
        'fnal_contribution_plus_de_10_salaries',
        'fnal_contribution_moins_de_20_salaries',
        'fnal_contribution_plus_de_20_salaries',
        'fnal_contribution_moins_de_50_salaries',
        'fnal_contribution_plus_de_50_salaries',
        'fnal_cotisation',
        ]

    commun.children.update(autres.fin_syndic.children)  # À harmoniser !
    if autres.fin_syndic.metadata is not None and autres.fin_syndic.metadata.get('order') is not None:
        commun.metadata['order'] += autres.fin_syndic.metadata['order']

    # Enlève de commun.metadata['order'] les paramètres qui ne font pas partie de commun.
    for id in commun.metadata['order'].copy():
        if id not in commun.children.keys():
            commun.metadata['order'].remove(id)

    # Réindexation Non Cadre
    # Initialisation
    noncadre = ParameterNode('noncadre', data=dict(
        description='Cotisations employeur pour salarié non cadre',
        metadata=dict(order=[]),
        ))
    pat.add_child('noncadre', noncadre)
    pat.children['noncadre'].children.update(retraites.agff.employeur.noncadre.children)
    pat.children['noncadre'].children.update(retraites.arrco.taux_effectifs_salaries_employeurs.employeur.noncadre.children)
    pat.children['noncadre'].children.update(retraites.ceg.employeur.children)
    pat.children['noncadre'].children.update(retraites.cet2019.employeur.children)
    pat.children['noncadre'].children.update(retraites.agirc_arrco.employeur.children)
    pat.children['noncadre'].children.update(commun.children)
    pat.children['noncadre'].metadata['order'] += commun.metadata['order']

    # Réindexation Cadre
    # Initialisation
    cadre = ParameterNode('cadre', data=dict(
        description='Cotisations employeur pour salarié cadre',
        metadata=dict(order=[]),
        ))
    pat.add_child('cadre', cadre)
    pat.children['cadre'].children.update(retraites.agff.employeur.cadre.children)
    pat.children['cadre'].children.update(retraites.arrco.taux_effectifs_salaries_employeurs.employeur.cadre.children)
    pat.children['cadre'].children.update(retraites.agirc.taux_effectifs_salaries_employeurs.avant81.employeur.children)
    pat.children['cadre'].children.update(retraites.apec.employeur.children)
    pat.children['cadre'].children.update(retraites.ceg.employeur.children)
    pat.children['cadre'].children.update(retraites.cet2019.employeur.children)
    pat.children['cadre'].children.update(retraites.agirc_arrco.employeur.children)
    del pat.children['cadre'].children['forfait_annuel']
    pat.children['cadre'].children.update(retraites.cet.employeur.children)
    pat.children['cadre'].children.update(commun.children)
    pat.children['cadre'].metadata['order'] += commun.metadata['order']

    # Réindexation Fonc
    # Initialisation
    fonc = ParameterNode('fonc', data=dict(description='Cotisations sociales employeur du secteur public'))
    pat.add_child('fonc', fonc)
    fonc.add_child('colloc', ParameterNode('colloc', data=dict(
        description='Cotisations sociales employeur du secteur public pour les collectivités locales',
        metadata=dict(order=[]),
        )))
    fonc.add_child('etat', ParameterNode('etat', data=dict(
        description="Cotisations sociales employeur du secteur public pour les fonctions d'Etat",
        metadata=dict(order=[]),
        )))
    fonc.add_child('militaire', ParameterNode('militaire', data=dict(
        description='Cotisations sociales employeur du secteur public pour les militaires',
        metadata=dict(order=[]),
        )))
    fonc.add_child('contract', ParameterNode('contract', data=dict(
        description='Cotisations sociales employeur du secteur public pour les agents contractuels',
        metadata=dict(order=[]),
        )))

    # Contractuel
    pat.children['fonc'].children['contract'] = public.ircantec.taux_cotisations_appeles.employeur
    pat.children['fonc'].children['contract'].children.update(commun.children)
    pat.children['fonc'].children['contract'].metadata['order'] += commun.metadata['order']

    # Etat
    pat.children['fonc'].children['etat'].children.update(public.mmid.etat.tout_traitement.employeur.children)
    # pat.children['fonc'].children['etat'].children.update(public.mmid.etat.sous_plafond.employeur.children)
    pat.children['fonc'].children['etat'].children.update(public.rafp.employeur.children)
    pat.children['fonc'].children['etat'].children.update(public.retraite.ati.children)
    pat.children['fonc'].children['etat'].children.update(public.retraite.pension.employeur.civils.children)

    # Militaires
    pat.children['fonc'].children['militaire'].children.update(public.mmid.etat.tout_traitement.employeur.children)
    # pat.children['fonc'].children['militaire'].children.update(public.mmid.etat.sous_plafond.employeur.children)
    pat.children['fonc'].children['militaire'].children.update(public.rafp.employeur.children)
    pat.children['fonc'].children['militaire'].children.update(public.retraite.pension.employeur.militaires.children)

    # Collectivités Locales
    pat.children['fonc'].children['colloc'].children['hospitaliere'] = public.cnracl.employeur.hospitaliere
    pat.children['fonc'].children['colloc'].children['territoriale'] = public.cnracl.employeur.territoriale
    pat.children['fonc'].children['colloc'].children.update(public.cnracl.employeur.children)
    pat.children['fonc'].children['colloc'].children.update(public.mmid.colloc.tout_traitement.employeur.children)
    # pat.children['fonc'].children['colloc'].children.update(public.mmid.colloc.sous_plafond.employeur.children)
    pat.children['fonc'].children['colloc'].children.update(public.rafp.employeur.children)

    # Renaming
    pat.children['prive_non_cadre'] = pat.children.pop('noncadre')
    pat.children['prive_cadre'] = pat.children.pop('cadre')

    # Rework commun to deal with public employees
    for var in ['apprentissage_taxe', 'apprentissage_contribution_additionnelle', 'apprentissage_taxe_alsace_moselle', 'chomage', 'asf', 'ags',
                'construction_plus_de_10_salaries', 'construction_plus_de_20_salaries', 'construction_plus_de_50_salaries', 'maladie',
                'formprof_moins_de_10_salaries', 'formprof_moins_de_11_salaries', 'formprof_20_salaries_et_plus', 'formprof_11_salaries_et_plus', 'formprof_entre_10_et_19_salaries',
                'vieillesse_deplafonnee', 'vieillesse_plafonnee']:
        del commun.children[var]
        if var in commun.metadata['order']:
            commun.metadata['order'].remove(var)

    for var in ['apprentissage_taxe', 'apprentissage_contribution_additionnelle', 'apprentissage_taxe_alsace_moselle',
                'formprof_moins_de_10_salaries', 'formprof_moins_de_11_salaries', 'formprof_20_salaries_et_plus', 'formprof_11_salaries_et_plus', 'formprof_entre_10_et_19_salaries',
                'ags', 'construction_plus_de_10_salaries', 'construction_plus_de_20_salaries', 'construction_plus_de_50_salaries', 'chomage', 'asf']:
        del pat.children['fonc'].children['contract'].children[var]
        if var in pat.children['fonc'].children['contract'].metadata['order']:
            pat.children['fonc'].children['contract'].metadata['order'].remove(var)

    pat.children['fonc'].children['etat'].children.update(commun.children)
    pat.children['fonc'].children['etat'].metadata['order'] += commun.metadata['order']
    pat.children['fonc'].children['colloc'].children.update(commun.children)
    pat.children['fonc'].children['colloc'].metadata['order'] += commun.metadata['order']
    pat.children['fonc'].children['militaire'].children.update(commun.children)
    pat.children['fonc'].children['militaire'].metadata['order'] += commun.metadata['order']

    pat.children['etat_t'] = pat.children['fonc'].children['etat']  # Il semble que ce soient des sauvegardes temporaires ?
    pat.children['colloc_t'] = pat.children['fonc'].children['colloc']
    pat.children['contract'] = pat.children['fonc'].children['contract']
    pat.children['militaire_t'] = pat.children['fonc'].children['militaire']

    for var in ['etat', 'colloc', 'contract', 'militaire']:
        del pat.children['fonc'].children[var]

    # Renaming
    pat.children['public_titulaire_etat'] = pat.children.pop('etat_t')
    pat.children['public_titulaire_militaire'] = pat.children.pop('militaire_t')
    # del pat.children['public_titulaire_etat'].children['rafp']
    pat.children['public_titulaire_territoriale'] = pat.children.pop('colloc_t')
    pat.children['public_titulaire_hospitaliere'] = copy.deepcopy(pat.children['public_titulaire_territoriale'])
    for category in ['territoriale', 'hospitaliere']:
        for name, bareme in pat.children['public_titulaire_' + category].children[category].children.items():
            pat.children['public_titulaire_{}'.format(category)].children[name] = bareme

    for category in ['territoriale', 'hospitaliere']:
        del pat.children['public_titulaire_territoriale'].children[category]
        del pat.children['public_titulaire_hospitaliere'].children[category]

    pat.children['public_non_titulaire'] = pat.children.pop('contract')
    return pat


def build_sal(node_json):
    '''
    Construit le dictionnaire de barèmes des cotisations salariales
    '''
    sal = ParameterNode('sal', data=dict(description='Cotisations sociales salariales'))  # Génère sal
    commun = ParameterNode('commun', data=dict(
        description='Cotisations sociales salariales communes à plusieurs régimes',
        metadata=dict(order=[]),
        ))  # Génère commun

    # Réindexation: nouveaux chemins
    retraites = node_json.prelevements_sociaux.regimes_complementaires_retraite_secteur_prive
    chomage = node_json.prelevements_sociaux.cotisations_regime_assurance_chomage
    regime_general = node_json.prelevements_sociaux.cotisations_securite_sociale_regime_general
    public = node_json.prelevements_sociaux.cotisations_secteur_public
    indep = node_json.prelevements_sociaux.cotisations_taxes_independants_artisans_commercants
    liberal = node_json.prelevements_sociaux.professions_liberales

    # Création de commun
    commun.children.update(chomage.chomage.salarie.children)
    if chomage.chomage.salarie.metadata is not None and chomage.chomage.salarie.metadata.get('order') is not None:
        commun.metadata['order'] += chomage.chomage.salarie.metadata['order']

    commun.children.update(chomage.asf.salarie.children)
    if chomage.asf.salarie.metadata is not None and chomage.asf.salarie.metadata.get('order') is not None:
        commun.metadata['order'] += chomage.asf.salarie.metadata['order']

    commun.children.update(regime_general.mmid.salarie.children)
    del commun.children['reduction_plus_65_ans']
    if regime_general.mmid.salarie.metadata is not None and regime_general.mmid.salarie.metadata.get('order') is not None:
        commun.metadata['order'] += regime_general.mmid.salarie.metadata['order']

    commun.children.update(regime_general.mmid_am.children)
    del commun.children['allocations_chomage_et_preretraite']
    del commun.children['avantages_vieillesse']
    if regime_general.mmid_am.metadata is not None and regime_general.mmid_am.metadata.get('order') is not None:
        commun.metadata['order'] += regime_general.mmid_am.metadata['order']

    commun.children.update(regime_general.cnav.salarie.children)
    if regime_general.cnav.salarie.metadata is not None and regime_general.cnav.salarie.metadata.get('order') is not None:
        commun.metadata['order'] += regime_general.cnav.salarie.metadata['order']

    # Enlève de commun.metadata['order'] les paramètres qui ne font pas partie de commun.
    for id in commun.metadata['order'].copy():
        if id not in commun.children.keys():
            commun.metadata['order'].remove(id)

    # Non Cadre
    # Initialisation
    noncadre = ParameterNode('noncadre', data=dict(
        description='Cotisations salariales pour salarié non cadre',
        metadata=dict(order=[]),
        ))
    sal.add_child('noncadre', noncadre)
    sal.children['noncadre'].children.update(retraites.agff.salarie.noncadre.children)
    sal.children['noncadre'].children.update(retraites.arrco.taux_effectifs_salaries_employeurs.salarie.noncadre.children)
    sal.children['noncadre'].children.update(retraites.ceg.salarie.children)
    sal.children['noncadre'].children.update(retraites.cet2019.salarie.children)
    sal.children['noncadre'].children.update(retraites.agirc_arrco.salarie.children)
    sal.children['noncadre'].children.update(commun.children)
    sal.children['noncadre'].metadata['order'] += commun.metadata['order']

    # Cadre
    cadre = ParameterNode('cadre', data=dict(
        description='Cotisations salariales pour salarié cadre',
        metadata=dict(order=[]),
        ))
    sal.add_child('cadre', cadre)
    sal.children['cadre'].children.update(retraites.agff.salarie.cadre.children)
    sal.children['cadre'].children.update(retraites.arrco.taux_effectifs_salaries_employeurs.salarie.cadre.children)
    sal.children['cadre'].children.update(retraites.agirc.taux_effectifs_salaries_employeurs.avant81.salarie.children)
    sal.children['cadre'].children.update(retraites.apec.salarie.children)
    sal.children['cadre'].children.update(retraites.ceg.salarie.children)
    sal.children['cadre'].children.update(retraites.cet2019.salarie.children)
    sal.children['cadre'].children.update(retraites.agirc_arrco.salarie.children)
    del sal.children['cadre'].children['forfait_annuel']

    sal.children['cadre'].children.update(retraites.cet.salarie.children)
    sal.children['cadre'].children.update(commun.children)
    sal.children['cadre'].metadata['order'] += commun.metadata['order']

    # Renaming
    sal.children['prive_non_cadre'] = sal.children.pop('noncadre')
    sal.children['prive_cadre'] = sal.children.pop('cadre')

    # Réindexation Fonc
    # Initialisation
    fonc = ParameterNode('fonc', data=dict(description='Cotisations salariales du secter public'))
    sal.add_child('fonc', fonc)
    fonc.add_child('colloc', ParameterNode('colloc', data=dict(
        description='Cotisations sociales salariales du secteur public pour les collectivités locales',
        metadata=dict(order=[]),
        )))
    fonc.add_child('etat', ParameterNode('etat', data=dict(
        description="Cotisations sociales salariales du secteur public pour les fonctions d'Etat",
        metadata=dict(order=[]),
        )))
    fonc.add_child('contract', ParameterNode('contract', data=dict(
        description='Cotisations sociales salariales du secteur public pour les agents contractuels',
        metadata=dict(order=[]),
        )))
    fonc.add_child('commun', ParameterNode('commun', data=dict(
        description='Cotisations sociales salariales communes à plusieurs régimes',
        metadata=dict(order=[]),
        )))

    # Etat
    sal.children['fonc'].children['etat'].children.update(public.rafp.salarie.children)
    sal.children['fonc'].children['etat'].children.update(public.retraite.pension.salarie.children)
    # sal.children['fonc'].children['colloc'].children.update(public.mmid.etat.tout_traitement.salarie.children)
    # sal.children['fonc'].children['colloc'].children.update(public.mmid.etat.sous_plafond.salarie.children)
    sal.children['public_titulaire_etat'] = sal.children['fonc'].children['etat']

    # Collectivités Locales
    sal.children['fonc'].children['colloc'].children.update(public.cnracl.salarie.children)
    # sal.children['fonc'].children['colloc'].children.update(public.mmid.colloc.tout_traitement.salarie.children)
    # sal.children['fonc'].children['colloc'].children.update(public.mmid.colloc.sous_plafond.salarie.children)
    sal.children['public_titulaire_territoriale'] = sal.children['fonc'].children['colloc']
    sal.children['public_titulaire_hospitaliere'] = sal.children['fonc'].children['colloc']

    # Contractuel
    sal.children['fonc'].children['contract'] = public.ircantec.taux_cotisations_appeles.salarie
    sal.children['public_non_titulaire'] = sal.children['fonc'].children['contract']

    # Commun
    sal.children['fonc'].children['commun'].children.update(public.fds.salarie.children)  # À harmoniser ! + Créer params depuis IPP

    for type_sal_category in [
            'public_titulaire_etat',
            'public_titulaire_territoriale',
            'public_titulaire_hospitaliere',
            'public_non_titulaire',
            ]:
        sal.children[type_sal_category].children['excep_solidarite'] = sal.children['fonc'].children['commun'].children['solidarite']

    # Ajoute le RAFP (Régime additionnel de la fonction publique) pour 'public_titulaire_territoriale' et 'public_titulaire_hospitaliere'
    for type_sal_category in ['public_titulaire_territoriale', 'public_titulaire_hospitaliere']:
        sal.children[type_sal_category].children['rafp'] = sal.children['fonc'].children['etat'].children['rafp']
    sal.children['public_non_titulaire'].children.update(commun.children)
    sal.children['public_non_titulaire'].metadata['order'] += commun.metadata['order']
    for var in ['chomage', 'asf']:
        del sal.children['public_non_titulaire'].children[var]
        if sal.children['public_non_titulaire'].metadata['order']:
            sal.children['public_non_titulaire'].metadata['order'].remove(var)

    # Cleaning
    del sal.children['fonc'].children['etat']
    del sal.children['fonc'].children['colloc']
    del sal.children['fonc'].children['contract']

    # Arti
    sal.add_child('arti', ParameterNode('arti', data=dict(description='Cotisations sociales salariales des artisans')))
    sal.children['arti'].children.update(indep.famille.arti.children)
    sal.children['arti'].children.update(indep.mmid.arti.children)
    # Comind
    sal.add_child('comind', ParameterNode('comind', data=dict(description='Cotisations sociales salariales des commercants et indépendants')))
    sal.children['comind'].children.update(indep.famille.comind.children)
    sal.children['comind'].children.update(indep.mmid.comind.children)
    # Microsocial
    sal.add_child('microsocial', ParameterNode('microsocial', data=dict(description='Cotisations sociales salariales des professions libérales')))
    sal.children['microsocial'].children.update(liberal.auto_entrepreneur.children)  # À harmoniser ! + Créer params depuis IPP

    return sal


def preprocess_parameters(parameters):
    '''
    Preprocess the legislation parameters to build the cotisations sociales taxscales (barèmes)
    '''
    pat = build_pat(parameters)
    sal = build_sal(parameters)

    cotsoc = ParameterNode('cotsoc', data=dict(description='Cotisations sociales'))
    parameters.add_child('cotsoc', cotsoc)
    cotsoc.add_child('pat', pat)
    cotsoc.add_child('sal', sal)

    # Modifs
    cotsoc.add_child('cotisations_employeur', ParameterNode('cotisations_employeur_after_preprocessing', data=dict(description='Cotisations sociales employeur')))
    cotsoc.add_child('cotisations_salarie', ParameterNode('cotisations_salarie_after_preprocessing', data=dict(description='Cotisations sociales salariales')))

    for cotisation_name, baremes in (
            ('cotisations_employeur', pat.children),
            ('cotisations_salarie', sal.children),
            ):
        for category, bareme in baremes.items():
            if category in [member.name for member in TypesCategorieSalarie]:
                cotsoc.children[cotisation_name].children[category] = bareme

    return parameters



from copy import deepcopy

from openfisca_core import periods
from openfisca_core.tools import assert_near

from .test_entities import TEST_CASE_AGES, new_simulation
from .cache import tax_benefit_system


def test_nb_enfants():
    test_case = deepcopy(TEST_CASE_AGES)
    year = 2013
    test_case['period'] = year
    simulation = tax_benefit_system.new_scenario().init_from_dict(test_case).new_simulation()
    from openfisca_france.model.prestations.prestations_familiales.base_ressource import nb_enf

    month = periods.period(year).first_month
    assert_near(nb_enf(simulation.famille, month, 3, 18), [2, 0])
    assert_near(nb_enf(simulation.famille, month, 19, 50), [0, 1])  # Adults don't count

    test_case['individus']['ind5']['autonomie_financiere'] = True
    simulation_2 = new_simulation(test_case)

    assert_near(nb_enf(simulation_2.famille, month, 19, 50), [0, 0])



import datetime

from openfisca_core import periods
from openfisca_core.reforms import Reform
from openfisca_core.tools import assert_near

from openfisca_france.scenarios import init_single_entity

from ..cache import tax_benefit_system

simulation_year = 2013
simulation_period = periods.period(simulation_year)


def modify_parameters(parameters):
    parameters.impot_revenu.bareme_ir_depuis_1945.bareme[0].rate.update(period=simulation_period, value=1)
    return parameters


class ir_100_tranche_1(Reform):
    name = "Imposition à 100% dès le premier euro et jusqu'à la fin de la 1ère tranche"

    def apply(self):
        self.modify_parameters(modifier_function = modify_parameters)


def test_parametric_reform():
    reform = ir_100_tranche_1(tax_benefit_system)

    scenario = init_single_entity(reform.new_scenario(),
        axes = [[
            dict(
                count = 3,
                name = 'salaire_imposable',
                max = 100000,
                min = 0,
                ),
            ]],
        period = simulation_period,
        parent1 = dict(date_naissance = datetime.date(simulation_year - 40, 1, 1)),
        )

    reference_simulation = scenario.new_simulation(use_baseline = True)
    assert_near(reference_simulation.calculate('impots_directs', period = simulation_period), [0, -7889.20019531, -23435.52929688],
        absolute_error_margin = 1)

    reform_simulation = scenario.new_simulation()
    assert_near(
        reform_simulation.calculate('impots_directs', simulation_period),
        [0, -13900.20019531, -29446.52929688],
        absolute_error_margin = 1,
        )


if __name__ == '__main__':
    import logging
    import sys
    logging.basicConfig(level = logging.ERROR, stream = sys.stdout)
    test_parametric_reform()



import os
import csv
import codecs
import pkg_resources
import openfisca_france
from numpy import fromiter
from openfisca_france.model.base import *


# Importe les barèmes locaux de la taxe d'habitation


def preload_parametres_locaux_taxe_habitation(year = None, variable_to_load = None):
    assert variable_to_load is not None
    assert year is not None
    if os.path.isfile('{}/assets/taxe_habitation/parametres_th_{}.csv'.format(openfisca_france.__name__, year)):
        with pkg_resources.resource_stream(
                openfisca_france.__name__,
                'assets/taxe_habitation/parametres_th_{}.csv'.format(year),
                ) as csv_file:
            utf8_reader = codecs.getreader('utf-8')
            csv_reader = csv.DictReader(utf8_reader(csv_file))
            return {
                row['code_insee_commune']: row[variable_to_load]
                for row in csv_reader
                }


class taux_th_commune(Variable):
    value_type = float
    default_value = 0
    entity = Menage
    label = "Taux de taxe d'habitation de la commune"
    definition_period = YEAR

    def formula(menage, period):
        code_INSEE_commune = menage('depcom', period.first_month)
        annee = period.start.offset('first-of', 'year').year
        taux_by_com = preload_parametres_locaux_taxe_habitation(year = annee, variable_to_load = 'taux_com')
        default_value = 0
        if taux_by_com is None:
            montant = fromiter((default_value for code_INSEE_commune_cell in code_INSEE_commune), dtype = float)
        else:
            montant = fromiter(
                (
                    taux_by_com.get(code_INSEE_commune_cell if isinstance(code_INSEE_commune_cell, str) else code_INSEE_commune_cell.decode('utf-8'), default_value)
                    for code_INSEE_commune_cell in code_INSEE_commune
                    ),
                dtype = float,
                )
        return montant


class taux_th_epci(Variable):
    value_type = float
    default_value = 0
    entity = Menage
    label = "Taux de taxe d'habitation de l'EPCI (Établissement Public de Coopération Intercommunale)"
    definition_period = YEAR

    def formula(menage, period):
        code_INSEE_commune = menage('depcom', period.first_month)
        annee = period.start.offset('first-of', 'year').year
        taux_by_epci = preload_parametres_locaux_taxe_habitation(year = annee, variable_to_load = 'taux_epci')
        default_value = 0
        if taux_by_epci is None:
            montant = fromiter((default_value for code_INSEE_commune_cell in code_INSEE_commune), dtype = float)
        else:
            montant = fromiter(
                (
                    taux_by_epci.get(code_INSEE_commune_cell if isinstance(code_INSEE_commune_cell, str) else code_INSEE_commune_cell.decode('utf-8'), default_value)
                    for code_INSEE_commune_cell in code_INSEE_commune
                    ),
                dtype = float,
                )
        return montant


class valeur_locative_moyenne_th_commune(Variable):
    value_type = float
    default_value = 0
    entity = Menage
    label = "Valeur locative moyenne utilisée pour le calcul des abattements de la taxe d'habitation de la commune"
    definition_period = YEAR

    def formula(menage, period):
        code_INSEE_commune = menage('depcom', period.first_month)
        annee = period.start.offset('first-of', 'year').year
        valeur_locative_moyenne_by_com = preload_parametres_locaux_taxe_habitation(year = annee, variable_to_load = 'valeur_locative_moyenne_com')
        default_value = 0
        if valeur_locative_moyenne_by_com is None:
            montant = fromiter((default_value for code_INSEE_commune_cell in code_INSEE_commune), dtype = float)
        else:
            montant = fromiter(
                (
                    valeur_locative_moyenne_by_com.get(code_INSEE_commune_cell if isinstance(code_INSEE_commune_cell, str) else code_INSEE_commune_cell.decode('utf-8'), default_value)
                    for code_INSEE_commune_cell in code_INSEE_commune
                    ),
                dtype = float,
                )
        return montant


class valeur_locative_moyenne_th_epci(Variable):
    value_type = float
    default_value = 0
    entity = Menage
    label = "Valeur locative moyenne utilisée pour le calcul des abattements de la taxe d'habitation de l'EPCI (Établissement Public de Coopération Intercommunale)"
    definition_period = YEAR

    def formula(menage, period):
        code_INSEE_commune = menage('depcom', period.first_month)
        annee = period.start.offset('first-of', 'year').year
        valeur_locative_moyenne_by_epci = preload_parametres_locaux_taxe_habitation(year = annee, variable_to_load = 'valeur_locative_moyenne_epci')
        default_value = 0
        if valeur_locative_moyenne_by_epci is None:
            montant = fromiter((default_value for code_INSEE_commune_cell in code_INSEE_commune), dtype = float)
        else:
            montant = fromiter(
                (
                    valeur_locative_moyenne_by_epci.get(code_INSEE_commune_cell if isinstance(code_INSEE_commune_cell, str) else code_INSEE_commune_cell.decode('utf-8'), default_value)
                    for code_INSEE_commune_cell in code_INSEE_commune
                    ),
                dtype = float,
                )
        return montant


class abt_general_base_th_commune(Variable):
    value_type = float
    default_value = 0
    entity = Menage
    label = "Quotité (ajustée) de l'abattement général à la base - taxe d'habitation de la commune"
    definition_period = YEAR

    def formula(menage, period):
        code_INSEE_commune = menage('depcom', period.first_month)
        annee = period.start.offset('first-of', 'year').year
        abt_general_base_by_com = preload_parametres_locaux_taxe_habitation(year = annee, variable_to_load = 'abt_general_base_com')
        default_value = 0
        if abt_general_base_by_com is None:
            montant = fromiter((default_value for code_INSEE_commune_cell in code_INSEE_commune), dtype = float)
        else:
            montant = fromiter(
                (
                    abt_general_base_by_com.get(code_INSEE_commune_cell if isinstance(code_INSEE_commune_cell, str) else code_INSEE_commune_cell.decode('utf-8'), default_value)
                    for code_INSEE_commune_cell in code_INSEE_commune
                    ),
                dtype = float,
                )
        return montant


class abt_general_base_th_epci(Variable):
    value_type = float
    default_value = 0
    entity = Menage
    label = "Quotité (ajustée) de l'abattement général à la base - taxe d'habitation de l'EPCI (Établissement Public de Coopération Intercommunale)"
    definition_period = YEAR

    def formula(menage, period):
        code_INSEE_commune = menage('depcom', period.first_month)
        annee = period.start.offset('first-of', 'year').year
        abt_general_base_by_epci = preload_parametres_locaux_taxe_habitation(year = annee, variable_to_load = 'abt_general_base_epci')
        default_value = 0
        if abt_general_base_by_epci is None:
            montant = fromiter((default_value for code_INSEE_commune_cell in code_INSEE_commune), dtype = float)
        else:
            montant = fromiter(
                (
                    abt_general_base_by_epci.get(code_INSEE_commune_cell if isinstance(code_INSEE_commune_cell, str) else code_INSEE_commune_cell.decode('utf-8'), default_value)
                    for code_INSEE_commune_cell in code_INSEE_commune
                    ),
                dtype = float,
                )
        return montant


class abt_pac_1_2_th_commune(Variable):
    value_type = float
    default_value = 0
    entity = Menage
    label = "Quotité (ajustée) de l'abattement obligatoire pour charges de famille, pour les deux premières personnes à charge - taxe d'habitation de la commune"
    definition_period = YEAR

    def formula(menage, period):
        code_INSEE_commune = menage('depcom', period.first_month)
        annee = period.start.offset('first-of', 'year').year
        abt_pac_1_2_by_com = preload_parametres_locaux_taxe_habitation(year = annee, variable_to_load = 'abt_pac_1_2_com')
        default_value = 0
        if abt_pac_1_2_by_com is None:
            montant = fromiter((default_value for code_INSEE_commune_cell in code_INSEE_commune), dtype = float)
        else:
            montant = fromiter(
                (
                    abt_pac_1_2_by_com.get(code_INSEE_commune_cell if isinstance(code_INSEE_commune_cell, str) else code_INSEE_commune_cell.decode('utf-8'), default_value)
                    for code_INSEE_commune_cell in code_INSEE_commune
                    ),
                dtype = float,
                )
        return montant


class abt_pac_1_2_th_epci(Variable):
    value_type = float
    default_value = 0
    entity = Menage
    label = "Quotité (ajustée) de l'abattement obligatoire pour charges de famille, pour les deux premières personnes à charge - taxe d'habitation de l'EPCI (Établissement Public de Coopération Intercommunale)"
    definition_period = YEAR

    def formula(menage, period):
        code_INSEE_commune = menage('depcom', period.first_month)
        annee = period.start.offset('first-of', 'year').year
        abt_pac_1_2_by_epci = preload_parametres_locaux_taxe_habitation(year = annee, variable_to_load = 'abt_pac_1_2_epci')
        default_value = 0
        if abt_pac_1_2_by_epci is None:
            montant = fromiter((default_value for code_INSEE_commune_cell in code_INSEE_commune), dtype = float)
        else:
            montant = fromiter(
                (
                    abt_pac_1_2_by_epci.get(code_INSEE_commune_cell if isinstance(code_INSEE_commune_cell, str) else code_INSEE_commune_cell.decode('utf-8'), default_value)
                    for code_INSEE_commune_cell in code_INSEE_commune
                    ),
                dtype = float,
                )
        return montant


class abt_pac_3pl_th_commune(Variable):
    value_type = float
    default_value = 0
    entity = Menage
    label = "Quotité (ajustée) de l'abattement obligatoire pour charges de famille, pour les personnes à charge à partir de la troisième - taxe d'habitation de la commune"
    definition_period = YEAR

    def formula(menage, period):
        code_INSEE_commune = menage('depcom', period.first_month)
        annee = period.start.offset('first-of', 'year').year
        abt_pac_3pl_by_com = preload_parametres_locaux_taxe_habitation(year = annee, variable_to_load = 'abt_pac_3pl_com')
        default_value = 0
        if abt_pac_3pl_by_com is None:
            montant = fromiter((default_value for code_INSEE_commune_cell in code_INSEE_commune), dtype = float)
        else:
            montant = fromiter(
                (
                    abt_pac_3pl_by_com.get(code_INSEE_commune_cell if isinstance(code_INSEE_commune_cell, str) else code_INSEE_commune_cell.decode('utf-8'), default_value)
                    for code_INSEE_commune_cell in code_INSEE_commune
                    ),
                dtype = float,
                )
        return montant


class abt_pac_3pl_th_epci(Variable):
    value_type = float
    default_value = 0
    entity = Menage
    label = "Quotité (ajustée) de l'abattement obligatoire pour charges de famille, pour les personnes à charge à partir de la troisième - taxe d'habitation de l'EPCI (Établissement Public de Coopération Intercommunale)"
    definition_period = YEAR

    def formula(menage, period):
        code_INSEE_commune = menage('depcom', period.first_month)
        annee = period.start.offset('first-of', 'year').year
        abt_pac_3pl_by_epci = preload_parametres_locaux_taxe_habitation(year = annee, variable_to_load = 'abt_pac_3pl_epci')
        default_value = 0
        if abt_pac_3pl_by_epci is None:
            montant = fromiter((default_value for code_INSEE_commune_cell in code_INSEE_commune), dtype = float)
        else:
            montant = fromiter(
                (
                    abt_pac_3pl_by_epci.get(code_INSEE_commune_cell if isinstance(code_INSEE_commune_cell, str) else code_INSEE_commune_cell.decode('utf-8'), default_value)
                    for code_INSEE_commune_cell in code_INSEE_commune
                    ),
                dtype = float,
                )
        return montant


class abt_condition_modeste_th_commune(Variable):
    value_type = float
    default_value = 0
    entity = Menage
    label = "Quotité (ajustée) de l'abattement pour personnes de condition modeste - taxe d'habitation de la commune"
    definition_period = YEAR

    def formula(menage, period):
        code_INSEE_commune = menage('depcom', period.first_month)
        annee = period.start.offset('first-of', 'year').year
        abt_condition_modeste_by_com = preload_parametres_locaux_taxe_habitation(year = annee, variable_to_load = 'abt_condition_modeste_com')
        default_value = 0
        if abt_condition_modeste_by_com is None:
            montant = fromiter((default_value for code_INSEE_commune_cell in code_INSEE_commune), dtype = float)
        else:
            montant = fromiter(
                (
                    abt_condition_modeste_by_com.get(code_INSEE_commune_cell if isinstance(code_INSEE_commune_cell, str) else code_INSEE_commune_cell.decode('utf-8'), default_value)
                    for code_INSEE_commune_cell in code_INSEE_commune
                    ),
                dtype = float,
                )
        return montant


class abt_condition_modeste_th_epci(Variable):
    value_type = float
    default_value = 0
    entity = Menage
    label = "Quotité (ajustée) de l'abattement pour personnes de condition modeste - taxe d'habitation de l'EPCI (Établissement Public de Coopération Intercommunale)"
    definition_period = YEAR

    def formula(menage, period):
        code_INSEE_commune = menage('depcom', period.first_month)
        annee = period.start.offset('first-of', 'year').year
        abt_condition_modeste_by_epci = preload_parametres_locaux_taxe_habitation(year = annee, variable_to_load = 'abt_condition_modeste_epci')
        default_value = 0
        if abt_condition_modeste_by_epci is None:
            montant = fromiter((default_value for code_INSEE_commune_cell in code_INSEE_commune), dtype = float)
        else:
            montant = fromiter(
                (
                    abt_condition_modeste_by_epci.get(code_INSEE_commune_cell if isinstance(code_INSEE_commune_cell, str) else code_INSEE_commune_cell.decode('utf-8'), default_value)
                    for code_INSEE_commune_cell in code_INSEE_commune
                    ),
                dtype = float,
                )
        return montant



from openfisca_france.model.base import Individu, Variable, MONTH, \
    set_input_dispatch_by_period, round_, where


class chomage_cotisation_retraite_complementaire_journaliere(Variable):
    value_type = float
    entity = Individu
    label = 'Cotisation de retraite complémentaire journalière sur allocation chômage'
    definition_period = MONTH
    set_input = set_input_dispatch_by_period
    reference = [
        'https://www.unedic.org/indemnisation/fiches-thematiques/cumul-allocation-salaire',
        'https://www.unedic.org/indemnisation/fiches-thematiques/retenues-sociales-sur-les-allocations'
        ]

    def formula(individu, period, parameters):
        allocation_retour_emploi_journaliere = individu('allocation_retour_emploi_journaliere', period)
        are_salaire_journalier_reference = individu('are_salaire_journalier_reference', period)

        # le seuil d'exonération de cette cotisation est indexé sur le montant minimum d'ARE
        seuil_exoneration = parameters(period).chomage.allocation_retour_emploi.montant_minimum_hors_mayotte

        taux_cotisation = parameters(period).prelevements_sociaux.regimes_complementaires_retraite_secteur_prive.cotisation_retraite_complementaire.chomage.taux
        cotisation_theorique = are_salaire_journalier_reference * taux_cotisation
        allocation_cotisation_deduite = allocation_retour_emploi_journaliere - cotisation_theorique

        return round_(
            where(
                allocation_cotisation_deduite > seuil_exoneration,
                -1 * cotisation_theorique,
                0),
            2)



import logging
import os
import shutil
import yaml


PARENT_DIRECTORY = os.path.abspath('../..')
PATH_LENGTH_TO_IGNORE = len(PARENT_DIRECTORY)

PARAMETERS_DIRECTORY = os.path.join(PARENT_DIRECTORY, 'parameters')
PATH_MAX_LENGTH = 150

INDEX_FILENAME = 'index.yaml'

# Create logger
logging.basicConfig(level = logging.INFO)
logger = logging.getLogger(__name__)


def clean_from_filename(relative_path):
    last_separator_index = relative_path.rfind(os.path.sep)
    if last_separator_index == -1:
        raise ValueError('Directory expected but none found in: ' + relative_path)
    return relative_path[:last_separator_index]


def list_long_paths(absolute_directory_path):
    long_paths = []

    for directory, _sub_directories, files in os.walk(absolute_directory_path):
        for f in files:
            file_path = os.path.join(directory, f)
            if (len(file_path) - PATH_LENGTH_TO_IGNORE) > PATH_MAX_LENGTH:
                path_extract = clean_from_filename(get_sub_parent_path(file_path, PARAMETERS_DIRECTORY))
                if path_extract not in long_paths:
                    long_paths.append(path_extract)

    return long_paths


def get_sub_parent_path(directory_path, parent_path):
    absolute_path = os.path.abspath(directory_path)
    return os.path.relpath(absolute_path, parent_path)


def get_raw_content(yaml_filepath):
    with open(yaml_filepath, 'r') as stream:
        return yaml.load(stream)


def harvest(item):
    '''
    Put into a dictionary the content of the file hierarchy starting at the given item.

    An 'index.yaml' file content is directly added to the current dictionary key.
    Any other file or a directory content in stored under a new key (file or directory name).

    @param item: A directory of yaml files or a yaml file.
    '''
    content = {}
    item_name = os.path.basename(item)

    if os.path.isdir(item):
        sub_items = os.listdir(item)
        sub_content = {}

        # Harvest index.yaml first if it exists
        index_content = None
        if INDEX_FILENAME in sub_items:
            index_content = harvest(os.path.join(item, INDEX_FILENAME))
            sub_items.remove(INDEX_FILENAME)

        for subitem in sub_items:
            sub_content.update(harvest(os.path.join(item, subitem)))

        # Add index content on top
        if index_content:
            sub_content.update(index_content)

        content[item_name] = sub_content

    else:
        if item_name == INDEX_FILENAME:
            content.update(get_raw_content(item))
        else:
            # Clean file name from extension
            item_name = os.path.splitext(item_name)[0]
            # Store content under cleaned file name section
            content[item_name] = get_raw_content(item)

    return content


def parse_and_clean(directory, paths_to_clean):
    items = os.listdir(directory)

    # Parse items in directory and harvest content for long paths
    for item in items:
        item_path = os.path.join(directory, item)
        functional_path = get_sub_parent_path(item_path, PARAMETERS_DIRECTORY)

        if os.path.isdir(item_path):
            if functional_path in paths_to_clean:
                yaml_path = os.path.join(directory, item + '.yaml')

                logger.info(os.linesep + 'Cleaning long directory into: ' + yaml_path)
                content = harvest(item_path)
                with open(yaml_path, 'w') as yaml_file:
                    yaml_file.write(yaml.dump(content[item], default_flow_style=False, allow_unicode=True))

                # Delete harvested directory
                logger.info('Deleting ' + item_path)
                shutil.rmtree(item_path)

            else:
                parse_and_clean(item_path, paths_to_clean)


long_parameters_paths = list_long_paths(PARAMETERS_DIRECTORY)
logger.info('{} directories have files with more than {} characters in their paths starting from this directory: {}'.format(len(long_parameters_paths), PATH_MAX_LENGTH, PARENT_DIRECTORY).encode('utf-8'))

while len(long_parameters_paths) > 0:
    parse_and_clean(PARAMETERS_DIRECTORY, long_parameters_paths)
    long_parameters_paths = list_long_paths(PARAMETERS_DIRECTORY)



from openfisca_france.model.base import *
from openfisca_france.model.prestations.prestations_familiales.base_ressource import nb_enf


class aefa(Variable):
    '''
    Aide exceptionelle de fin d'année (prime de Noël)
    Instituée en 1998
    Apparaît sous le nom de complément de rmi dans les ERF
    Le montant de l’aide mentionnée à l’article 1er versée aux bénéficiaires de l’allocation de solidarité
    spécifique à taux majoré servie aux allocataires âgés de cinquante-cinq ans ou plus justifiant de vingt années
    d’activité salariée, aux allocataires âgés de cinquante-sept ans et demi ou plus justifiant de dix années d’activité
    salariée ainsi qu’aux allocataires justifiant d’au moins 160 trimestres validés dans les régimes d’assurance
    vieillesse ou de périodes reconnues équivalentes est égal à
    Pour bénéficier de la Prime de Noël 2011, vous devez être éligible pour le compte du mois de novembre 2011
    ou au plus de décembre 2011, soit d’une allocation de solidarité spécifique (ASS), de la prime forfaitaire mensuelle
    de reprise d'activité, de l'allocation équivalent retraite (allocataire AER), du revenu de solidarité active
    (Bénéficiaires RSA), de l'allocation de parent isolé (API), du revenu minimum d'insertion (RMI), de l’Allocation
    pour la Création ou la Reprise d'Entreprise (ACCRE-ASS) ou encore allocation chômage.
    '''
    value_type = float
    entity = Famille
    label = "Aide exceptionelle de fin d'année (prime de Noël)"
    reference = [
        'Décret n° 2022-1568 du 14 décembre 2022',
        'https://www.legifrance.gouv.fr/jorf/id/JORFTEXT000046738342'
        ]
    definition_period = YEAR

    def formula_2002_01_01(famille, period, parameters):
        rsa = famille('rsa', period, options = [ADD])
        ass_i = famille.members('ass', period, options = [ADD])
        ass = famille.sum(ass_i)
        api = famille('api', period, options = [ADD])
        aer_i = famille.members('aer', period, options = [ADD])
        aer = famille.sum(aer_i)
        condition = (ass > 0) + (aer > 0) + (api > 0) + (rsa > 0)
        condition_majoration = rsa > 0

        af = parameters(period).prestations_sociales.prestations_familiales.prestations_generales.af
        janvier = period.first_month
        af_nbenf = famille('af_nbenf', janvier)
        nb_parents = famille('nb_parents', janvier)
        if hasattr(af, 'age3'):
            nombre_enfants = nb_enf(famille, janvier, af.af_cm.age1, af.af_cm.age3)
        else:
            nombre_enfants = af_nbenf

        aefa = parameters(period).prestations_sociales.solidarite_insertion.autre_solidarite.aefa

        nombre_de_personnes_supplementaires_eligibles = (nb_parents >= 2) + nombre_enfants
        majoration = 1 + (condition_majoration * (
            (nombre_de_personnes_supplementaires_eligibles >= 1) * aefa.taux_majoration.tx_2p
            + min_(nombre_enfants, 2) * aefa.taux_majoration.tx_supp * (nb_parents == 2)
            + aefa.taux_majoration.tx_supp * (nb_parents == 1) * (nombre_enfants >= 2)
            + aefa.taux_majoration.tx_3pac * max_(nombre_enfants - 2, 0)
            ))

        montant_aefa = aefa.montant_prime * majoration
        montant_aefa += aefa.prime_exceptionnelle

        return condition * montant_aefa



from numpy import logical_or as or_

from openfisca_france.model.base import *
from openfisca_france.model.prestations.prestations_familiales.base_ressource import nb_enf


class af_nbenf(Variable):
    value_type = int
    entity = Famille
    label = "Nombre d'enfants dans la famille au sens des allocations familiales"
    definition_period = MONTH
    set_input = set_input_dispatch_by_period

    def formula(famille, period):
        prestations_familiales_enfant_a_charge_i = famille.members('prestations_familiales_enfant_a_charge', period)
        af_nbenf = famille.sum(prestations_familiales_enfant_a_charge_i)

        return af_nbenf


class af_coeff_garde_alternee(Variable):
    value_type = float
    default_value = 1
    entity = Famille
    label = 'Coefficient à appliquer aux af pour tenir compte de la garde alternée'
    definition_period = MONTH
    set_input = set_input_dispatch_by_period

    def formula_2007_05_01(famille, period):
        nb_enf = famille('af_nbenf', period)
        garde_alternee = famille.members('garde_alternee', period)
        pfam_enfant_a_charge = famille.members('prestations_familiales_enfant_a_charge', period)

        # Le nombre d'enfants à charge en garde alternée, qui vérifient donc pfam_enfant_a_charge = true et garde_alternee = true
        nb_enf_garde_alternee = famille.sum(garde_alternee * pfam_enfant_a_charge)

        # Avoid division by zero. If nb_enf == 0, necessarily nb_enf_garde_alternee = 0 so coeff = 1
        coeff = 1 - (nb_enf_garde_alternee / (nb_enf + (nb_enf == 0))) * 0.5

        return coeff


class af_allocation_forfaitaire_nb_enfants(Variable):
    value_type = int
    entity = Famille
    label = "Nombre d'enfants ouvrant droit à l'allocation forfaitaire des AF"
    definition_period = MONTH
    set_input = set_input_dispatch_by_period

    def formula(famille, period, parameters):
        af = parameters(period).prestations_sociales.prestations_familiales.prestations_generales.af
        af_forfaitaire_nbenf = nb_enf(famille, period, af.af_cm.age3, af.af_cm.age3)

        return af_forfaitaire_nbenf


class af_eligibilite_base(Variable):
    value_type = bool
    entity = Famille
    label = 'Allocations familiales - Éligibilité pour la France métropolitaine sous condition de ressources'
    definition_period = MONTH
    set_input = set_input_dispatch_by_period

    def formula(famille, period):
        residence_dom = famille.demandeur.menage('residence_dom', period)
        af_nbenf = famille('af_nbenf', period)

        return not_(residence_dom) * (af_nbenf >= 2)


class af_eligibilite_dom(Variable):
    value_type = bool
    entity = Famille
    label = 'Allocations familiales - Éligibilité pour les DOM (hors Mayotte) sous condition de ressources'
    definition_period = MONTH
    set_input = set_input_dispatch_by_period

    def formula(famille, period):
        residence_dom = famille.demandeur.menage('residence_dom', period)
        residence_mayotte = famille.demandeur.menage('residence_mayotte', period)
        af_nbenf = famille('af_nbenf', period)

        return residence_dom * not_(residence_mayotte) * (af_nbenf >= 1)


class af_base(Variable):
    value_type = float
    entity = Famille
    label = 'Allocations familiales - allocation de base'
    definition_period = MONTH
    set_input = set_input_divide_by_period
    # prestations familiales (brutes de crds)

    def formula(famille, period, parameters):
        eligibilite_base = famille('af_eligibilite_base', period)
        eligibilite_dom = famille('af_eligibilite_dom', period)
        af_nbenf = famille('af_nbenf', period)

        af = parameters(period).prestations_sociales.prestations_familiales.prestations_generales.af
        bmaf = parameters(period).prestations_sociales.prestations_familiales.bmaf.bmaf

        eligibilite = or_(eligibilite_base, eligibilite_dom)

        un_seul_enfant = (
            eligibilite_dom
            * (af_nbenf == 1)
            * af.af_maj_dom.allocations_familiales_un_enfant
            )

        deux_enfants = (af_nbenf >= 2) * af.af_cm.taux.enf2
        plus_de_trois_enfants = max_(af_nbenf - 2, 0) * af.af_cm.taux.enf3
        taux_total = un_seul_enfant + deux_enfants + plus_de_trois_enfants
        montant_base = eligibilite * round_(bmaf * taux_total, 2)
        coeff_garde_alternee = famille('af_coeff_garde_alternee', period)
        montant_base = montant_base * coeff_garde_alternee

        af_taux_modulation = famille('af_taux_modulation', period)
        montant_base_module = montant_base * af_taux_modulation

        return montant_base_module


class af_taux_modulation(Variable):
    value_type = float
    default_value = 1
    entity = Famille
    label = 'Taux de modulation à appliquer au montant des AF depuis 2015'
    definition_period = MONTH
    set_input = set_input_dispatch_by_period

    def formula_2015_07_01(famille, period, parameters):
        nb_enf_tot = famille('af_nbenf', period)

        return taux_helper(famille, period, parameters, nb_enf_tot)


class af_allocation_forfaitaire_taux_modulation(Variable):
    value_type = float
    default_value = 1
    entity = Famille
    label = "Taux de modulation à appliquer à l'allocation forfaitaire des AF depuis 2015"
    definition_period = MONTH
    set_input = set_input_dispatch_by_period

    def formula_2015_07_01(famille, period, parameters):
        af_nbenf = famille('af_nbenf', period)
        af_forfaitaire_nbenf = famille('af_allocation_forfaitaire_nb_enfants', period)
        nb_enf_tot = af_nbenf + af_forfaitaire_nbenf

        return taux_helper(famille, period, parameters, nb_enf_tot)


class af_age_aine(Variable):
    value_type = int
    default_value = AGE_INT_MINIMUM
    entity = Famille
    label = "Allocations familiales - Âge de l'aîné des enfants éligibles"
    definition_period = MONTH
    set_input = set_input_dispatch_by_period
    is_period_size_independent = True

    def formula(famille, period, parameters):
        af = parameters(period).prestations_sociales.prestations_familiales.prestations_generales.af
        age = famille.members('age', period)
        pfam_enfant_a_charge = famille.members('prestations_familiales_enfant_a_charge', period)
        condition_eligibilite = pfam_enfant_a_charge * (age <= af.af_cm.age2)
        age_enfants_eligiles = age * condition_eligibilite
        return max_(famille.max(age_enfants_eligiles, role = Famille.ENFANT), AGE_INT_MINIMUM)  # max_ may return -inf, so negative caping by AGE_INT_MINIMUM is needed


class af_majoration_enfant(Variable):
    value_type = float
    entity = Individu
    label = "Allocations familiales - Majoration pour âge applicable à l'enfant"
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(individu, period, parameters):
        pfam_enfant_a_charge = individu('prestations_familiales_enfant_a_charge', period)
        age = individu('age', period)
        garde_alternee = individu('garde_alternee', period)

        af_nbenf = individu.famille('af_nbenf', period)
        af_base = individu.famille('af_base', period)
        age_aine = individu.famille('af_age_aine', period)

        af = parameters(period).prestations_sociales.prestations_familiales.prestations_generales.af
        bmaf = parameters(period).prestations_sociales.prestations_familiales.bmaf.bmaf

        montant_enfant_seul = bmaf * (
            (af.af_maj_dom.tranches_age.age_debut_premiere_tranche <= age)
            * (age < af.af_maj_dom.tranches_age.age_debut_deuxieme_tranche)
            * af.af_maj_dom.majoration_premier_enfant.taux_tranche_1
            + (af.af_maj_dom.tranches_age.age_debut_deuxieme_tranche <= age)
            * af.af_maj_dom.majoration_premier_enfant.taux_tranche_2
            )

        montant_plusieurs_enfants = bmaf * (
            (af.af_maj.maj_age_deux_enfants.age1 <= age)
            * (age < af.af_maj.maj_age_deux_enfants.age2)
            * af.af_maj.maj_age_deux_enfants.taux1
            + (af.af_maj.maj_age_deux_enfants.age2 <= age)
            * af.af_maj.maj_age_deux_enfants.taux2
            )

        montant = (af_nbenf == 1) * montant_enfant_seul + (af_nbenf > 1) * montant_plusieurs_enfants

        # Attention ! Ne fonctionne pas pour les enfants du même âge (typiquement les jumeaux...)
        pas_aine = or_(af_nbenf != 2, (af_nbenf == 2) * not_(age == age_aine))

        coeff_garde_alternee = where(garde_alternee, af.af_cm.facteur_garde_alternee, 1)

        return (
            pfam_enfant_a_charge
            * (af_base > 0)
            * pas_aine
            * montant
            * coeff_garde_alternee
            )


class af_majoration(Variable):
    value_type = float
    entity = Famille
    label = 'Allocations familiales - majoration pour âge'
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(famille, period):
        af_majoration_enfant = famille.members('af_majoration_enfant', period)
        af_majoration_enfants_famille = famille.sum(af_majoration_enfant, role = Famille.ENFANT)

        af_taux_modulation = famille('af_taux_modulation', period)
        af_majoration_enfants_module = af_majoration_enfants_famille * af_taux_modulation

        return af_majoration_enfants_module


class af_complement_degressif(Variable):
    value_type = float
    entity = Famille
    label = 'AF - Complément dégressif en cas de dépassement du plafond'
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula_2015_07_01(famille, period, parameters):
        eligibilite_dom = famille('af_eligibilite_dom', period)
        af_base = famille('af_base', period)
        af_majoration = famille('af_majoration', period)
        af = af_base + af_majoration
        af_nbenf = famille('af_nbenf', period)
        depassement_mensuel = depassement_helper(famille, period, parameters, af_nbenf)

        return (
            not_(eligibilite_dom)
            * (depassement_mensuel > 0)
            * max_(0, af - depassement_mensuel)
            )


class af_allocation_forfaitaire_complement_degressif(Variable):
    value_type = float
    entity = Famille
    label = "AF - Complément dégressif pour l'allocation forfaitaire en cas de dépassement du plafond"
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula_2015_07_01(famille, period, parameters):
        af_nbenf = famille('af_nbenf', period)
        af_forfaitaire_nbenf = famille('af_allocation_forfaitaire_nb_enfants', period)
        nb_enf_tot = af_nbenf + af_forfaitaire_nbenf  # noqa F841

        depassement_mensuel = depassement_helper(famille, period, parameters, af_nbenf)

        af_allocation_forfaitaire = famille('af_allocation_forfaitaire', period)
        return max_(0, af_allocation_forfaitaire - depassement_mensuel) * (depassement_mensuel > 0)


class af_allocation_forfaitaire(Variable):
    value_type = float
    entity = Famille
    label = 'Allocations familiales - forfait'
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula_2003_07_01(famille, period, parameters):
        af_nbenf = famille('af_nbenf', period)
        af_forfaitaire_nbenf = famille('af_allocation_forfaitaire_nb_enfants', period)
        af = parameters(period).prestations_sociales.prestations_familiales.prestations_generales.af
        bmaf = parameters(period).prestations_sociales.prestations_familiales.bmaf.bmaf

        af_forfait = round_(bmaf * af.af_maj.majoration_enfants.allocation_forfaitaire.taux, 2)
        af_allocation_forfaitaire = ((af_nbenf >= 2) * af_forfaitaire_nbenf) * af_forfait

        af_forfaitaire_taux_modulation = famille('af_allocation_forfaitaire_taux_modulation', period)
        af_forfaitaire_module = af_allocation_forfaitaire * af_forfaitaire_taux_modulation

        return af_forfaitaire_module


class af(Variable):
    calculate_output = calculate_output_add
    value_type = float
    entity = Famille
    label = 'Allocations familiales - total des allocations'
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula_2015_07_01(famille, period, parameters):
        af_base = famille('af_base', period)
        af_majoration = famille('af_majoration', period)
        af_allocation_forfaitaire = famille('af_allocation_forfaitaire', period)
        af_complement_degressif = famille('af_complement_degressif', period)
        af_forfaitaire_complement_degressif = famille('af_allocation_forfaitaire_complement_degressif', period)

        return (
            af_base
            + af_majoration
            + af_allocation_forfaitaire
            + af_complement_degressif
            + af_forfaitaire_complement_degressif
            )

    def formula(famille, period, parameters):
        af_base = famille('af_base', period)
        af_majoration = famille('af_majoration', period)
        af_allocation_forfaitaire = famille('af_allocation_forfaitaire', period)

        return af_base + af_majoration + af_allocation_forfaitaire


def plafonds_helper(famille, period, parameters, nb_enf_tot):
    af = parameters(period).prestations_sociales.prestations_familiales.prestations_generales.af
    modulation = af.af_cond_ress

    plafond1 = (
        modulation.plafond_tranche_1_base
        + max_(nb_enf_tot, 0) * modulation.majoration_plafond_par_enfant_supplementaire
        )

    plafond2 = (
        modulation.plafond_tranche_2_base
        + max_(nb_enf_tot, 0) * modulation.majoration_plafond_par_enfant_supplementaire
        )

    return (plafond1, plafond2)


def taux_helper(famille, period, parameters, nb_enf_tot):
    af = parameters(period).prestations_sociales.prestations_familiales.prestations_generales.af
    modulation = af.af_cm.modulation

    base_ressources = famille('prestations_familiales_base_ressources', period)

    plafond1, plafond2 = plafonds_helper(famille, period, parameters, nb_enf_tot)

    taux = (
        (base_ressources <= plafond1) * 1
        + (base_ressources > plafond1) * (base_ressources <= plafond2) * modulation.taux_tranche_2
        + (base_ressources > plafond2) * modulation.taux_tranche_3
        )

    return taux


def depassement_helper(famille, period, parameters, nb_enf_tot):
    base_ressources = famille('prestations_familiales_base_ressources', period)

    plafond1, plafond2 = plafonds_helper(famille, period, parameters, nb_enf_tot)

    depassement_plafond1 = max_(0, base_ressources - plafond1)
    depassement_plafond2 = max_(0, base_ressources - plafond2)

    depassement_mensuel = (
        (depassement_plafond2 == 0) * depassement_plafond1
        + (depassement_plafond2 > 0) * depassement_plafond2
        ) / 12

    return depassement_mensuel



from ..model.base import *


def modify_parameters(parameters):
    parameters.marche_travail.salaire_minimum.smic.smic_b_horaire.update(period = period(2013), value = 9)
    return parameters


class smic_h_b_9_euros(Reform):
    name = 'Réforme pour simulation ACOSS SMIC horaire brut fixe à 9 euros'

    def apply(self):
        self.modify_parameters(modifier_function = modify_parameters)
